%ifarch x86_64
    %global kvm_target    x86_64
%endif
%ifarch aarch64
    %global kvm_target    aarch64
%endif
%ifarch i686
    %global kvm_target    i686
%endif
%global buildarch %{kvm_target}-softmmu

# Generate documentation
%global have_docs 0

# Generate position-independent executables; note that disabling
# this makes it difficult to patch QEMU
%global have_pie 1

# Support for loadable modules
%global have_modules 1

# Debug switches
# Generate debug info
%global have_debug_info 0
# Debug the Tiny Code Generator
%global have_tcg_debug 0
# Debug casts
%global have_qom_cast_debug 0
# Enable Sparse semantic checker
%global have_sparse 0
# Mutex debugging support
%global have_debug_mutex 0

# Support for gperftools
%global have_gperftools 0

# Build qemu-io, qemu-nbd and qemu-img tools
%global have_tools 1

# Support for capstone disassembler
%global have_capstone 0

# Support for vnuma
%global have_numa 1

# Support curses on the HMP interface
%global have_curses 0
%global have_iconv 0

# Support for Linux AIO
%global have_linux_aio 1

# Support Linux's vhost-net module
%global have_vhost_net 1

# Support virtio sockets for host/guest communication
%global have_vhost_vsock 1

# Support Linux's vhost-scsi module
%global have_vhost_scsi 1

# Support vhost-crypto acceleration
%global have_vhost_crypto 0

# Support vhost-user
%global have_vhost_user 1

# Support vhost-user-fs
%global have_vhost_user_fs 1

# Support vhost-kernel
%global have_vhost_kernel 1

# Support for replication
%global have_replication 1

# Support for xfsctl()
%global have_xfsctl 0

# Memory knobs
# Allocate a pool of memory for coroutines
%global have_coroutine_pool 1
# Thread-caching malloc
%global have_tcmalloc 0
# jemalloc
%global have_jemalloc 0
# membarrier system call (for Linux 4.14+ or Windows)
%global have_membarrier 0
# pmem support (note that there's no libpmem support on aarch64)
%ifarch x86_64
%global have_libpmem 1
%else
%global have_libpmem 0
%endif

# Security and capabilities checking knobs
# libseccomp
%global have_seccomp 1
# libattr
%global have_attr 1
# libcap-ng
%global have_libcapng 0

# Support for block storage backends
# iSCSI
%global have_iscsi 1
# Ceph/RBD
%global have_rbd 1
# GlusterFS
%global have_gluster 1
# VirtFS (required for Kata Containers)
%global have_virtfs 1
# SSH
%global have_ssh 1
# Curl
%global have_curl 1
# Block migration in the main migration stream
%global have_live_block_migration 1

# Support for image formats
# bochs
%global have_bochs 0
# cloop
%global have_cloop 0
# qcow1
%global have_qcow1 0
# vvfat
%global have_vvfat 0
# qed
%global have_qed 0
# Parallels
%global have_parallels 0
# Sheepdog
%global have_sheepdog 0
# MacOS DMG
%global have_dmg 0

# Support for multipath persistent reservations
%global have_mpath 0

# Support for remote graphical consoles
# VNC
%global have_vnc 1
# SASL encryption for VNC
%global have_vnc_sasl 1
# JPEG compression for VNC
%global have_vnc_jpg 0
# PNG compression for VNC
%global have_vnc_png 0
# Spice
%global have_spice 0

# Support for compression libraries
# LZO
%global have_lzo 0
# Snappy
%global have_snappy 0
# bzip
%global have_bzip2 0
# lzfse
%global have_lzfse 0

# Support for encryption libraries
# GNU TLS
%global have_gnutls 1
# Nettle
%global have_nettle 0
# libgrypt
%global have_gcrypt 0
# Linux AF_ALG crypto backend driver
%global have_crypto_afalg 1

# Miscellaneous networking support
# RDMA (for migration only)
%global have_rdma 0
# Virtual Distributed Ethernet (VDE)
%global have_vde 0
# netmap
%global have_netmap 0

# Support for miscellaneous hardware
# Trusted Platform Modules (TPM)
%global have_tpm 1
# USB redirection and passthrough
%global have_usb 0
# SmartCards
%global have_smartcard 0
# Flat Device Trees (required for aarch64)
%ifarch aarch64
%global have_fdt 1
%else
%global have_fdt 0
%endif
# HAX acceleration support
%global have_hax 0
# Hypervisor.framework acceleration support
%global have_hvf 0
# Windows Hypervisor Platform acceleration support
%global have_whpx 0
# Veritas HyperScale vDisk backend support
%global have_vxhs 0

# Support for guest graphics APIs
# SDL/GTK
%global have_sdl_gtk 0
# OpenGL
%global have_opengl 0
# virgl
%global have_virgl 0

# Build guest agents for Linux and Windows
%global have_agent 1
%global have_windows_agent 0

# Xen is available only on i386/x86_64 (from libvirt spec)
%ifarch x86_64
%global have_xen 0
%endif

# Support for xkbcommon
%global have_xkbcommon 0

# Include dependencies on EDK2 packages
%ifarch x86_64 aarch64
%global have_edk2 1
%endif

# QMP regdump tool (and sosreport plugin) is supported on x86_64 and aarch64
%ifarch x86_64 aarch64
%global have_qmpregdump 1
%endif

# Control whether to run Parfait static analysis on code base
%bcond_with parfait

%global requires_all_modules                                     \
%if %{have_iscsi}                                                \
Requires: %{name}-block-iscsi = %{epoch}:%{version}-%{release}   \
%endif                                                           \
%if %{have_gluster}                                              \
Requires: %{name}-block-gluster = %{epoch}:%{version}-%{release} \
%endif                                                           \
%if %{have_rbd}                                                  \
Requires: %{name}-block-rbd = %{epoch}:%{version}-%{release}     \
%endif                                                           \
%if %{have_curl}                                                 \
Requires: %{name}-block-curl = %{epoch}:%{version}-%{release}    \
%endif                                                           \
%if %{have_ssh}                                                  \
Requires: %{name}-block-ssh = %{epoch}:%{version}-%{release}     \
%endif

Summary: QEMU is a machine emulator and virtualizer
Name: qemu-kvm
Version: 4.2.1
Release: 27%{?dist}
Epoch: 15
License: GPLv2+ and LGPLv2+ and BSD
Group: Development/Tools
URL: http://www.qemu.org/
Obsoletes: qemu-kvm-ev, qemu-img-ev, qemu-kvm-common-ev
ExclusiveArch: x86_64 aarch64

# Note that the source tarball is still called "qemu" (instead of "qemu-kvm")
# to avoid breaking the automated build system
Source0: qemu-4.2.1.tar.xz

# Creates /dev/kvm
Source3: 80-kvm.rules

%if 0%{?have_agent}
# Guest agent service
Source9: qemu-guest-agent.service
# Guest agent service environment file
Source10: qemu-ga.sysconfig
# Guest agent udev rules
Source11: 99-qemu-guest-agent.rules
%endif

# /etc/qemu/bridge.conf
Source12: bridge.conf

# /etc/modprobe.d/kvm.conf
Source20: kvm.conf
# /etc/modprobe.d/vhost.conf
Source21: vhost.conf

%if 0%{?have_qmpregdump}
# /usr/bin/qmp-regdump
Source22: qmp-regdump
# /usr/lib/<python sitelib>/sos/report/plugins/qemu_regdump.py
Source23: qemu_regdump.py
%endif

# Parfait configuration file
Source24: parfait-qemu.conf

Patch0001: kvm-trace-use-STAP_SDT_V2-to-work-around-symbol-visibili.patch

# Generic build dependencies
BuildRequires: zlib-devel
BuildRequires: glib2-devel
BuildRequires: which
BuildRequires: gnutls-devel
BuildRequires: cyrus-sasl-devel
BuildRequires: libaio-devel
BuildRequires: libtool
BuildRequires: pciutils-devel
BuildRequires: python3-devel
BuildRequires: texinfo
BuildRequires: perl-podlators
BuildRequires: chrpath

# ACPI compilation on x86_64
%ifarch x86_64
BuildRequires: iasl
BuildRequires: cpp
%endif

%if 0%{?have_sdl_gtk}
# -display sdl support
BuildRequires: SDL2-devel
%endif
# qemu-pr-helper multipath support (requires libudev too)
%if 0%{?have_mpath}
BuildRequires: device-mapper-multipath-devel
%endif
BuildRequires: systemd-devel
%if 0%{?have_iscsi}
# iscsi drive support
BuildRequires: libiscsi-devel
%endif
%if 0%{?have_snappy}
# snappy compression for memory dump
BuildRequires: snappy-devel
%endif
%if 0%{?have_lzo}
# lzo compression for memory dump
BuildRequires: lzo-devel
%endif
%if 0%{?have_curses}
# needed for -display curses
BuildRequires: ncurses-devel
%endif
# used by 9pfs
%if 0%{?have_attr}
BuildRequires: libattr-devel
%endif
BuildRequires: libcap-devel
%if 0%{?have_libcapng}
# used by qemu-bridge-helper
BuildRequires: libcap-ng-devel
%endif
%if 0%{?have_spice}
# spice usb redirection support
BuildRequires: usbredir-devel >= 0.7.1
%endif
%ifnarch s390 s390x
%if 0%{?have_gperftools}
# tcmalloc support
BuildRequires: gperftools-devel
%endif
%endif
%if 0%{?have_spice}
# spice graphics support
BuildRequires: spice-protocol >= 0.12.2
BuildRequires: spice-server-devel >= 0.12.0
%endif
%if 0%{?have_seccomp}
# seccomp containment support
BuildRequires: libseccomp-devel >= 2.3.0
%endif
%if 0%{?have_rbd}
# For rbd block driver
BuildRequires: librbd1-devel
%endif
# We need both because the 'stap' binary is probed for by configure
BuildRequires: systemtap
BuildRequires: systemtap-sdt-devel
%if 0%{?have_vnc_jpg}
# For VNC JPEG support
BuildRequires: libjpeg-devel
%endif
%if 0%{?have_vnc_png}
# For VNC PNG support
BuildRequires: libpng-devel
%endif
# For uuid generation
BuildRequires: libuuid-devel
%if 0%{?have_fdt}
# For FDT device tree support
BuildRequires: libfdt-devel
%endif
# Hard requirement for version >= 1.3
BuildRequires: pixman-devel
%if 0%{?have_gluster}
# For gluster support
BuildRequires: glusterfs-devel >= 3.4.0
BuildRequires: glusterfs-api-devel >= 3.4.0
%endif
%if 0%{?have_usb}
# Needed for USB passthrough
BuildRequires: libusbx-devel >= 1.0.22
%endif
%if 0%{?have_ssh}
# SSH block driver
BuildRequires: libssh-devel
%endif
%if 0%{?have_curl}
# For network block driver
BuildRequires: libcurl-devel
%endif
%if 0%{?have_sdl_gtk}
# GTK frontend
BuildRequires: gtk3-devel
BuildRequires: vte291-devel
# GTK translations
BuildRequires: gettext
%endif
# RDMA migration
%if 0%{have_rdma}
BuildRequires: librdmacm-devel
%endif
%if 0%{?have_xen}
# Xen support
BuildRequires: xen-devel
%endif
%ifarch %{ix86} x86_64 aarch64
# qemu 2.1: needed for memdev hostmem backend
BuildRequires: numactl-devel
%endif
# qemu 2.3: reading bzip2 compressed dmg images
BuildRequires: bzip2-devel
%if 0%{?have_sdl_gtk}
# qemu 2.4: needed for opengl bits
BuildRequires: libepoxy-devel
%endif
# qemu 2.5: needed for TLS test suite
BuildRequires: libtasn1-devel
%if 0%{?have_smartcard}
# qemu 2.5: libcacard is it's own project now
BuildRequires: libcacard-devel >= 2.5.0
%endif
%if 0%{?have_virgl}
# qemu 2.5: virgl 3d support
BuildRequires: virglrenderer-devel
%endif
%if 0%{?have_sdl_gtk}
# qemu 2.6: Needed for gtk GL support
BuildRequires: mesa-libgbm-devel
%endif
%if 0%{?with_parfait}
# qemu 4.0: Needed for nproc(1)
BuildRequires: coreutils
BuildRequires: libnsl
%endif
%if 0%{?have_libpmem}
BuildRequires: libpmem-devel
%endif
%if 0%{?have_xkbcommon}
BuildRequires: libxkbcommon-devel
%endif

Requires: qemu-kvm-core = %{epoch}:%{version}-%{release}

%{requires_all_modules}

%description
qemu-kvm is an open source virtualizer that provides hardware
emulation for the KVM hypervisor. qemu-kvm acts as a virtual
machine monitor together with the KVM kernel modules, and emulates the
hardware for a full system such as a PC and its associated peripherals.


%package -n qemu-kvm-core
Summary: qemu-kvm core components
Requires: kernel-uek
Requires: qemu-img = %{epoch}:%{version}-%{release}
%ifarch x86_64
Requires: seabios-bin >= 1.10.2-1
Requires: sgabios-bin
Requires: edk2-ovmf
%endif
%ifarch aarch64
Requires: edk2-aarch64
%endif

Requires: %{name}-common = %{epoch}:%{version}-%{release}
Requires: libseccomp >= 2.3.0
# For compressed guest memory dumps
Requires: lzo snappy
%ifnarch aarch64
Requires: seavgabios-bin >= 1.10.2-1
Requires: ipxe-roms-qemu >= 20170123-1
%endif
%if 0%{?have_gluster}
Requires: glusterfs-api >= 3.6.0
%endif
%if 0%{?have_usb}
Requires: libusbx >= 1.0.19
Requires: usbredir >= 0.7.1
%endif
# Kata RPMs don't exist for OL8 and depend on OL7 naming scheme so fulfill
# dependency for now.
Provides: qemu

%description -n qemu-kvm-core
qemu-kvm is an open source virtualizer that provides hardware
emulation for the KVM hypervisor. qemu-kvm acts as a virtual
machine monitor together with the KVM kernel modules, and emulates the
hardware for a full system such as a PC and its associated peripherals.


%package -n qemu-img
Summary: QEMU command line tool for manipulating disk images
Group: Development/Tools

%description -n qemu-img
This package provides a command line tool for manipulating disk images.


%package -n qemu-kvm-common
Summary: QEMU common files needed by all QEMU targets
Group: Development/Tools
Requires(post): /usr/bin/getent
Requires(post): /usr/sbin/groupadd
Requires(post): /usr/sbin/useradd

%description -n qemu-kvm-common
qemu-kvm is an open source virtualizer that provides hardware emulation for
the KVM hypervisor.

This package provides documentation and auxiliary programs used with qemu-kvm.


%if 0%{?have_agent}
%package -n qemu-guest-agent
Summary: QEMU guest agent
Requires(post): systemd-units
Requires(preun): systemd-units
Requires(postun): systemd-units

%description -n qemu-guest-agent
qemu-kvm is an open source virtualizer that provides hardware emulation for
the KVM hypervisor.

This package provides an agent to run inside guests, which communicates
with the host over a virtio-serial channel named "org.qemu.guest_agent.0"

This package does not need to be installed on the host OS.
%endif


%if 0%{?have_gluster}
%package  block-gluster
Summary: QEMU Gluster block driver
Requires: %{name}-common%{?_isa} = %{epoch}:%{version}-%{release}
%description block-gluster
This package provides the additional Gluster block driver for QEMU.

Install this package if you want to access remote Gluster storage.
%endif


%if 0%{?have_iscsi}
%package  block-iscsi
Summary: QEMU iSCSI block driver
Requires: %{name}-common%{?_isa} = %{epoch}:%{version}-%{release}

%description block-iscsi
This package provides the additional iSCSI block driver for QEMU.

Install this package if you want to access iSCSI volumes.
%endif


%if 0%{?have_rbd}
%package  block-rbd
Summary: QEMU Ceph/RBD block driver
Requires: %{name}-common%{?_isa} = %{epoch}:%{version}-%{release}

%description block-rbd
This package provides the additional Ceph/RBD block driver for QEMU.

Install this package if you want to access remote Ceph volumes
using the rbd protocol.
%endif


%if 0%{?have_ssh}
%package  block-ssh
Summary: QEMU SSH block driver
Requires: %{name}-common%{?_isa} = %{epoch}:%{version}-%{release}

%description block-ssh
This package provides the additional SSH block driver for QEMU.

Install this package if you want to access remote disks using
the Secure Shell (SSH) protocol.
%endif


%if 0%{?have_curl}
%package  block-curl
Summary: QEMU CURL block driver
Requires: %{name}-common%{?_isa} = %{epoch}:%{version}-%{release}

%description block-curl
This package provides the additional CURL block driver for QEMU.

Install this package if you want to access remote disks over
http, https, ftp and other transports provided by the CURL library.
%endif


%prep
%setup -q -n qemu-%{version}%{?rcstr}
%autopatch -p1


%build


%if 0%{?have_spice}
    %global spiceflag --enable-spice
%else
    %global spiceflag --disable-spice
%endif

%if 0%{?have_iscsi}
    %global iscsiflag --enable-libiscsi
%else
    %global iscsiflag --disable-libiscsi
%endif

%if 0%{?have_smartcard}
    %global smartcardflag --enable-smartcard
%else
    %global smartcardflag --disable-smartcard
%endif

%if 0%{?have_fdt}
    %global fdtflag --enable-fdt
%else
    %global fdtflag --disable-fdt
%endif

%if 0%{?have_rbd}
    %global rbdflag --enable-rbd
%else
    %global rbdflag --disable-rbd
%endif

%if 0%{?have_sdl_gtk}
    %global sdlflag --enable-sdl --enable-sdl-image --enable-virglrenderer --enable-gtk --enable-vte
%else
    %global sdlflag --disable-sdl --disable-sdl-image --disable-virglrenderer --disable-gtk --disable-vte
%endif

%if 0%{?have_agent}
    %global guestflags --enable-guest-agent
%else
    %global guestflags --disable-guest-agent
%endif

%if 0%{?have_windows_agent}
    %global windowsguestflags --enable-guest-agent-msi
%else
    %global windowsguestflags --disable-guest-agent-msi
%endif

%if 0%{?have_tpm}
    %global tpmflags --enable-tpm
%else
    %global tpmflags --disable-tpm
%endif

%if 0%{?have_docs}
    %global docsflags --enable-docs
%else
    %global docsflags --disable-docs
%endif

%if 0%{?have_pie}
    %global pieflags --enable-pie
%else
    %global pieflags --disable-pie
%endif

%if 0%{?have_modules}
    %global moduleflags --enable-modules --enable-module-upgrades
%else
    %global moduleflags --disable-modules --disable-module-upgrades
%endif

%if 0%{?have_tcg_debug}
    %global tcgdebugflags --enable-debug-tcg
%else
    %global tcgdebugflags --disable-debug-tcg
%endif

%if 0%{?have_debug_info}
    %global debuginfoflags --enable-debug-info
%else
    %global debuginfoflags --disable-debug-info
%endif

%if 0%{?have_sparse}
    %global sparseflags --enable-sparse
%else
    %global sparseflags --disable-sparse
%endif

%if 0%{?have_gnutls}
    %global gnutlsflags --enable-gnutls
%else
    %global gnutlsflags --disable-gnutls
%endif

%if 0%{?have_nettle}
    %global nettleflags --enable-nettle
%else
    %global nettleflags --disable-nettle
%endif

%if 0%{?have_gcrypt}
    %global gcryptflags --enable-gcrypt
%else
    %global gcryptflags --disable-gcrypt
%endif

%if 0%{?have_curses}
    %global cursesflags --enable-curses
%else
    %global cursesflags --disable-curses
%endif

%if 0%{have_iconv}
    %global iconvflags --enable-iconv
%else
    %global iconvflags --disable-iconv
%endif

%if 0%{?have_vnc}
    %global vncflags --enable-vnc
%else
    %global vncflags --disable-vnc
%endif

%if 0%{?have_vnc_sasl}
    %global vncsaslflags --enable-vnc-sasl
%else
    %global vncsaslflags --disable-vnc-sasl
%endif

%if 0%{?have_vnc_jpg}
    %global vncjpgflags --enable-vnc-jpeg
%else
    %global vncjpgflags --disable-vnc-jpeg
%endif

%if 0%{?have_vnc_png}
    %global vncpngflags --enable-vnc-png
%else
    %global vncpngflags --disable-vnc-png
%endif

%if 0%{?have_virtfs}
    %global virtfsflags --enable-virtfs
%else
    %global virtfsflags --disable-virtfs
%endif

%if 0%{?have_xen}
    %global xenflags --enable-xen --enable-xen-pci-passthrough
%else
    %global xenflags --disable-xen --disable-xen-pci-passthrough
%endif

%if 0%{?have_rdma}
    %global rdmaflags --enable-rdma --enable-pvrdma
%else
    %global rdmaflags --disable-rdma --disable-pvrdma
%endif

%if 0%{?have_vde}
    %global vdeflags --enable-vde
%else
    %global vdeflags --disable-vde
%endif

%if 0%{?have_netmap}
    %global netmapflags --enable-netmap
%else
    %global netmapflags --disable-netmap
%endif

%if 0%{?have_linux_aio}
    %global linuxaioflags --enable-linux-aio
%else
    %global linuxaioflags --disable-linux-aio
%endif

%if 0%{?have_libcapng}
    %global libcapngflags --enable-cap-ng
%else
    %global libcapngflags --disable-cap-ng
%endif

%if 0%{?have_attr}
    %global attrflags --enable-attr
%else
    %global attrflags --disable-attr
%endif

%if 0%{?have_vhost_net}
    %global vhostnetflags --enable-vhost-net
%else
    %global vhostnetflags --disable-vhost-net
%endif

%if 0%{?have_usb}
    %global usbflags --enable-libusb --enable-usb-redir
%else
    %global usbflags --disable-libusb --disable-usb-redir
%endif

%if 0%{?have_lzo}
    %global lzoflags --enable-lzo
%else
    %global lzoflags --disable-lzo
%endif

%if 0%{?have_snappy}
    %global snappyflags --enable-snappy
%else
    %global snappyflags --disable-snappy
%endif

%if 0%{?have_bzip2}
    %global bzip2flags --enable-bzip2
%else
    %global bzip2flags --disable-bzip2
%endif

%if 0%{?have_lzfse}
    %global lzfseflags --enable-lzfse
%else
    %global lzfseflags --disable-lzfse
%endif

%if 0%{?have_seccomp}
    %global seccompflags --enable-seccomp
%else
    %global seccompflags --disable-seccomp
%endif

%if 0%{?have_coroutine_pool}
    %global coroutineflags --enable-coroutine-pool
%else
    %global coroutineflags --disable-coroutine-pool
%endif

%if 0%{?have_gluster}
    %global glusterflags --enable-glusterfs
%else
    %global glusterflags --disable-glusterfs
%endif

%if 0%{?have_ssh}
    %global sshflags --enable-libssh
%else
    %global sshflags --disable-libssh
%endif

%if 0%{?have_curl}
    %global curlflags --enable-curl
%else
    %global curlflags --disable-curl
%endif

%if 0%{?have_numa}
    %global numaflags --enable-numa
%else
    %global numaflags --disable-numa
%endif

%if 0%{?have_tcmalloc}
    %global tcmallocflags --enable-tcmalloc
%else
    %global tcmallocflags --disable-tcmalloc
%endif

%if 0%{?have_jemalloc}
    %global jemallocflags --enable-jemalloc
%else
    %global jemallocflags --disable-jemalloc
%endif

%if 0%{?have_replication}
    %global replicationflags --enable-replication
%else
    %global replicationflags --disable-replication
%endif

%if 0%{?have_vhost_vsock}
    %global vhostvsockflags --enable-vhost-vsock
%else
    %global vhostvsockflags --disable-vhost-vsock
%endif

%if 0%{?have_vhost_scsi}
    %global vhostscsiflags --enable-vhost-scsi
%else
    %global vhostscsiflags --disable-vhost-scsi
%endif

%if 0%{?have_opengl}
    %global openglflags --enable-opengl
%else
    %global openglflags --disable-opengl
%endif

%if 0%{?have_virgl}
    %global virglflags --enable-virglrenderer
%else
    %global virglflags --disable-virglrenderer
%endif

%if 0%{?have_xfsctl}
    %global xfsctlflags --enable-xfsctl
%else
    %global xfsctlflags --disable-xfsctl
%endif

%if 0%{?have_mpath}
    %global mpathflags --enable-mpath
%else
    %global mpathflags --disable-mpath
%endif

%if 0%{?have_capstone}
    %global capstoneflags --enable-capstone
%else
    %global capstoneflags --disable-capstone
%endif

%if 0%{?have_qom_cast_debug}
    %global qomcastflags --enable-qom-cast-debug
%else
    %global qomcastflags --disable-qom-cast-debug
%endif

%if 0%{?have_membarrier}
    %global membarrierflags --enable-membarrier
%else
    %global membarrierflags --disable-membarrier
%endif

%if 0%{?have_libpmem}
    %global libpmemflags --enable-libpmem
%else
    %global libpmemflags --disable-libpmem
%endif

%if 0%{?have_hax}
    %global haxflags --enable-hax
%else
    %global haxflags --disable-hax
%endif

%if 0%{?have_hvf}
    %global hvfflags --enable-hvf
%else
    %global hvfflags --disable-hvf
%endif

%if 0%{?have_whpx}
    %global whpxflags --enable-whpx
%else
    %global whpxflags --disable-whpx
%endif

%if 0%{?have_vhost_crypto}
    %global vhostcyptoflags --enable-vhost-crypto
%else
    %global vhostcyptoflags --disable-vhost-crypto
%endif

%if 0%{?have_live_block_migration}
    %global liveblockmigrationflags --enable-live-block-migration
%else
    %global liveblockmigrationflags --disable-live-block-migration
%endif

%if 0%{have_bochs}
    %global bochsflags --enable-bochs
%else
    %global bochsflags --disable-bochs
%endif

%if 0%{have_cloop}
    %global cloopflags --enable-cloop
%else
    %global cloopflags --disable-cloop
%endif

%if 0%{have_qcow1}
    %global qcow1flags --enable-qcow1
%else
    %global qcow1flags --disable-qcow1
%endif

%if 0%{have_vvfat}
    %global vvfatflags --enable-vvfat
%else
    %global vvfatflags --disable-vvfat
%endif

%if 0%{have_qed}
    %global qedflags --enable-qed
%else
    %global qedflags --disable-qed
%endif

%if 0%{have_parallels}
    %global parallelsflags --enable-parallels
%else
    %global parallelsflags --disable-parallels
%endif

%if 0%{have_sheepdog}
    %global sheepdogflags --enable-sheepdog
%else
    %global sheepdogflags --disable-sheepdog
%endif

%if 0%{have_dmg}
    %global dmgflags --enable-dmg
%else
    %global dmgflags --disable-dmg
%endif

%if 0%{?have_tools}
    %global toolsflags --enable-tools
%else
    %global toolsflags --disable-tools
%endif

%if 0%{?have_crypto_afalg}
    %global cryptoafalgflags --enable-crypto-afalg
%else
    %global cryptoafalgflags --disable-crypto-afalg
%endif

%if 0%{?have_vhost_user}
    %global vhostuserflags --enable-vhost-user
%else
    %global vhostuserflags --disable-vhost-user
%endif

%if 0%{?have_vhost_user_fs}
   %global vhostuserfsflags --enable-vhost-user-fs
%else
   %global vhostuserfsflags --disable-vhost-user-fs
%endif

%if 0%{?have_vhost_kernel}
    %global vhostkernelflags --enable-vhost-kernel
%else
    %global vhostkernelflags --disable-vhost-kernel
%endif

%if 0%{?have_debug_mutex}
    %global debugmutexflags --enable-debug-mutex
%else
    %global debugmutexflags --disable-debug-mutex
%endif

%if 0%{?have_vxhs}
    %global vxhsflags --enable-vxhs
%else
    %global vxhsflags --disable-vxhs
%endif

%if 0%{?have_xkbcommon}
   %global xkbcommonflags --enable-xkbcommon
%else
   %global xkbcommonflags --disable-xkbcommon
%endif

%global block_drivers_list qcow2,raw,file,host_device,nbd,blkdebug,luks,null-co,nvme,copy-on-read,throttle

%if 0%{?have_gluster}
    %global block_drivers_list %{block_drivers_list},gluster
%endif
%if 0%{?have_iscsi}
    %global block_drivers_list %{block_drivers_list},iscsi
%endif
%if 0%{?have_rbd}
    %global block_drivers_list %{block_drivers_list},rbd
%endif
%if 0%{?have_vxhs}
    %global block_drivers_list %{block_drivers_list},vxhs
%endif
%if 0%{?have_ssh}
    %global block_drivers_list %{block_drivers_list},ssh
%endif
%if 0%{?have_curl}
    %global block_drivers_list %{block_drivers_list},curl
%endif

%global extra_ldflags -Wl,--build-id -specs=/usr/lib/rpm/redhat/redhat-hardened-ld

%ifnarch aarch64
    %global extra_ldflags %{extra_ldflags} -pie -Wl,-z,relro -Wl,-z,now
%endif

%global optflags %(echo %{optflags} | sed 's/-Wp,-D_FORTIFY_SOURCE=2//')
%global extra_cflags %{optflags} -specs=/usr/lib/rpm/redhat/redhat-hardened-cc1

%if 0%{?with_parfait}
    %global extra_cflags ${extra_cflags} -B parfait-ld
%endif

# --build-id option is used for giving info to the debug packages.
buildldflags="VL_LDFLAGS=-Wl,--build-id"

./configure \
    --prefix=%{_prefix} \
    --libdir=%{_libdir} \
    --sysconfdir=%{_sysconfdir} \
    --interp-prefix=%{_prefix}/qemu-%%M \
    --localstatedir=%{_localstatedir} \
    --docdir="%{qemudocdir}" \
    --libexecdir=%{_libexecdir} \
    --with-pkgversion=%{name}-%{version}-%{release} \
    --with-confsuffix=/"%{name}" \
    --firmwarepath=%{_prefix}/share/qemu-firmware \
    --extra-ldflags="%{extra_ldflags}" \
    --extra-cflags="%{extra_cflags}" \
    --python=%{__python3} \
    --target-list="%{buildarch}" \
    --with-coroutine=ucontext \
    --block-drv-rw-whitelist=%{block_drivers_list} \
    --block-drv-ro-whitelist=vmdk,vhdx,vpc,https,ssh \
    --tls-priority=@QEMU,SYSTEM \
    --disable-strip \
    --enable-werror \
    --enable-kvm \
    --enable-trace-backend=dtrace \
    --disable-user \
    --disable-linux-user \
    --disable-bsd-user \
    --disable-libnfs \
    --disable-brlapi \
    --disable-bluez \
    --audio-drv-list="" \
    %{?with_parfait:--disable-avx2} \
    %{?with_parfait:--cc=parfait-gcc} \
    %{?with_parfait:--cxx=parfait-g++} \
    %{spiceflag} \
    %{iscsiflag} \
    %{smartcardflag} \
    %{fdtflag} \
    %{rbdflag} \
    %{sdlflag} \
    %{guestflags} \
    %{windowsguestflags} \
    %{tpmflags} \
    %{docsflags} \
    %{pieflags} \
    %{moduleflags} \
    %{tcgdebugflags} \
    %{debuginfoflags} \
    %{sparseflags} \
    %{gnutlsflags} \
    %{nettleflags} \
    %{gcryptflags} \
    %{cursesflags} \
    %{iconvflags} \
    %{vncflags} \
    %{vncsaslflags} \
    %{vncjpgflags} \
    %{vncpngflags} \
    %{virtfsflags} \
    %{xenflags} \
    %{rdmaflags} \
    %{vdeflags} \
    %{netmapflags} \
    %{linuxaioflags} \
    %{libcapngflags} \
    %{attrflags} \
    %{vhostnetflags} \
    %{usbflags} \
    %{lzoflags} \
    %{snappyflags} \
    %{bzip2flags} \
    %{lzfseflags} \
    %{seccompflags} \
    %{coroutineflags} \
    %{glusterflags} \
    %{sshflags} \
    %{curlflags} \
    %{numaflags} \
    %{tcmallocflags} \
    %{jemallocflags} \
    %{replicationflags} \
    %{vhostvsockflags} \
    %{vhostscsiflags} \
    %{openglflags} \
    %{virglflags} \
    %{xfsctlflags} \
    %{qomcastflags} \
    %{mpathflags} \
    %{capstoneflags} \
    %{membarrierflags} \
    %{libpmemflags} \
    %{haxflags} \
    %{hvfflags} \
    %{whpxflags} \
    %{vhostcyptoflags} \
    %{liveblockmigrationflags} \
    %{bochsflags} \
    %{cloopflags} \
    %{qcow1flags} \
    %{vvfatflags} \
    %{qedflags} \
    %{parallelsflags} \
    %{sheepdogflags} \
    %{dmgflags} \
    %{toolsflags} \
    %{cryptoafalgflags} \
    %{vhostuserflags} \
    %{vhostuserfsflags} \
    %{vhostkernelflags} \
    %{debugmutexflags} \
    %{vxhsflags} \
    %{xkbcommonflags}

echo "config-host.mak contents:"
echo "==="
cat config-host.mak
echo "==="

make V=1 %{?_smp_mflags} $buildldflags

# Setup back compat qemu-kvm binary
%{__python3} scripts/tracetool.py --backend dtrace --format stap --group=all \
  --binary %{_libexecdir}/qemu-kvm --target-name %{kvm_target} \
  --target-type system --probe-prefix \
  qemu.kvm trace-events-all > qemu-kvm.stp

%{__python3} scripts/tracetool.py --backend dtrace --format simpletrace-stap \
  --group=all --binary %{_libexecdir}/qemu-kvm --target-name %{kvm_target} \
  --target-type system --probe-prefix \
  qemu.kvm trace-events-all > qemu-kvm-simpletrace.stp

# Ugly Red Hat creation that results in packaging the QEMU executable as
# /usr/libexec/qemu-kvm instead of upstream's /usr/bin/qemu-system-<arch>
cp -a %{kvm_target}-softmmu/qemu-system-%{kvm_target} qemu-kvm

#
# Run Parfait static analysis tool against specified baseline
# if '--with parfait' is passed to rpmbuild.
#
# http://parfait.us.oracle.com/
#
%if %{with parfait}
# URL to use for Parfait baseline
_parfait_server=%{?_parfait_server}%{!?_parfait_server:http://ca-qa-parfait.us.oracle.com:9990/parfait-server/projects/QEMU/tasks/v%{version}}
# number of jobs to run in parallel (default: value returned by nproc(1).)
_parfait_threads=%{?_parfait_threads}%{!?_parfait_threads:$(nproc)}
# Location to store graphical report (default: ./parfait_html in BUILD dir)
_parfait_output=%{?_parfait_output}%{!?_parfait_output:"./parfait_html"}
# Location of Parfait configuration rules file
_parfait_conf=%{_sourcedir}/parfait-qemu.conf
#
# Select whether to compare against baseline (default) using '-b'
# or upload results (-s) if _parfait_upload is set to 1 on command-line
# (if _parfait_upload is set, the the source base dir is also set via -z)
#
_parfait_upload=%{?_parfait_upload:"-s"}%{!?_parfait_upload:"-b"}
_parfait_srcdir=%{?_parfait_upload:"-z %{_builddir}/%{name}-%{version}/"}

RC=0
parfait -j $_parfait_threads -c $_parfait_conf -g $_parfait_output $_parfait_upload $_parfait_server $_parfait_srcdir . || RC=$?

#
# Work is ongoing to triage all the issues reported by Parfait on the QEMU
# codebase but until they are all are either fixed or marked as a
# False Positive/Will-Not-Fix on the QEMU Parfait server ($_parfait_server),
# the parfait run will return 1 so we need to check the return value
# and ignore it if it's 1 or zero (success, in which case this check can be
# removed).
#
# For more details on exit codes, see the Parfait documentation:
# http://parfait.us.oracle.com/latest/ParfaitInfo.html#exit-codes
#
if [ ${RC} != 0 -a ${RC} != 1 ]; then
    exit ${RC}
fi
%endif	# with parfait

%install
%define _udevdir %(pkg-config --variable=udevdir udev)
%define _udevrulesdir %{_udevdir}/rules.d

install -D -p -m 0644 %{_sourcedir}/vhost.conf $RPM_BUILD_ROOT%{_sysconfdir}/modprobe.d/vhost.conf
%ifarch x86_64
    install -D -p -m 0644 %{_sourcedir}/kvm.conf $RPM_BUILD_ROOT%{_sysconfdir}/modprobe.d/kvm.conf
%endif

mkdir -p $RPM_BUILD_ROOT%{_bindir}/
mkdir -p $RPM_BUILD_ROOT%{_udevrulesdir}/
mkdir -p $RPM_BUILD_ROOT%{_datadir}/%{name}
mkdir -p $RPM_BUILD_ROOT%{_sysconfdir}/%{name}

install -m 0644 %{_sourcedir}/80-kvm.rules $RPM_BUILD_ROOT%{_udevrulesdir}/80-kvm.rules

make DESTDIR=$RPM_BUILD_ROOT \
    sharedir="%{_datadir}/%{name}" \
    datadir="%{_datadir}/%{name}" \
    install

mkdir -p $RPM_BUILD_ROOT%{_datadir}/systemtap/tapset

%if 0%{?have_agent}
# Install qemu-guest-agent service and udev rules
mkdir -p $RPM_BUILD_ROOT%{_unitdir}
mkdir -p $RPM_BUILD_ROOT%{_sysconfdir}/sysconfig/qemu-ga
install -m 0644 %{_sourcedir}/qemu-guest-agent.service $RPM_BUILD_ROOT%{_unitdir}
install -m 0644 %{_sourcedir}/qemu-ga.sysconfig $RPM_BUILD_ROOT%{_sysconfdir}/sysconfig/qemu-ga
install -m 0644 %{_sourcedir}/99-qemu-guest-agent.rules $RPM_BUILD_ROOT%{_udevrulesdir}
# Install the fsfreeze hook script
install -D --preserve-timestamps \
            scripts/qemu-guest-agent/fsfreeze-hook \
            $RPM_BUILD_ROOT%{_sysconfdir}/qemu-ga/fsfreeze-hook
# Create the user script directory
mkdir -p $RPM_BUILD_ROOT%{_sysconfdir}/qemu-ga/fsfreeze-hook.d
# Install the fsfreeze sample scripts
mkdir -p $RPM_BUILD_ROOT%{_datadir}/%{name}/qemu-ga/fsfreeze-hook.d/
install --preserve-timestamps -m 0644 \
             scripts/qemu-guest-agent/fsfreeze-hook.d/*.sample \
             $RPM_BUILD_ROOT%{_datadir}/%{name}/qemu-ga/fsfreeze-hook.d/
# Create the log directory
mkdir -p $RPM_BUILD_ROOT%{_localstatedir}/log/qemu-ga/
# Install the qemu-ga binary
mkdir -p $RPM_BUILD_ROOT%{_bindir}
install -m 0755 qemu-ga ${RPM_BUILD_ROOT}%{_bindir}/qemu-ga
%endif

install -m 0755 qemu-kvm $RPM_BUILD_ROOT%{_libexecdir}/
install -m 0644 qemu-kvm.stp $RPM_BUILD_ROOT%{_datadir}/systemtap/tapset/
install -m 0644 qemu-kvm-simpletrace.stp $RPM_BUILD_ROOT%{_datadir}/systemtap/tapset/

rm $RPM_BUILD_ROOT%{_bindir}/qemu-system-%{kvm_target}
rm $RPM_BUILD_ROOT%{_datadir}/systemtap/tapset/qemu-system-%{kvm_target}.stp
rm $RPM_BUILD_ROOT%{_datadir}/systemtap/tapset/qemu-system-%{kvm_target}-simpletrace.stp
rm $RPM_BUILD_ROOT%{_datadir}/systemtap/tapset/qemu-system-%{kvm_target}-log.stp

# Install simpletrace
install -m 0755 scripts/simpletrace.py $RPM_BUILD_ROOT%{_datadir}/%{name}/simpletrace.py
# Avoid ambiguous 'python' interpreter name
sed -i -e '1 s/python/python3/' $RPM_BUILD_ROOT%{_datadir}/%{name}/simpletrace.py
mkdir -p $RPM_BUILD_ROOT%{_datadir}/%{name}/tracetool
install -m 0644 -t $RPM_BUILD_ROOT%{_datadir}/%{name}/tracetool scripts/tracetool/*.py
mkdir -p $RPM_BUILD_ROOT%{_datadir}/%{name}/tracetool/backend
install -m 0644 -t $RPM_BUILD_ROOT%{_datadir}/%{name}/tracetool/backend scripts/tracetool/backend/*.py
mkdir -p $RPM_BUILD_ROOT%{_datadir}/%{name}/tracetool/format
install -m 0644 -t $RPM_BUILD_ROOT%{_datadir}/%{name}/tracetool/format scripts/tracetool/format/*.py

%if 0%{?have_docs}
mkdir -p $RPM_BUILD_ROOT%{qemudocdir}
install -p -m 0644 -t ${RPM_BUILD_ROOT}%{qemudocdir} Changelog README COPYING COPYING.LIB LICENSE docs/interop/qmp-spec.txt
chmod -x ${RPM_BUILD_ROOT}%{_mandir}/man1/*
chmod -x ${RPM_BUILD_ROOT}%{_mandir}/man8/*
%endif

install -D -p -m 0644 qemu.sasl $RPM_BUILD_ROOT%{_sysconfdir}/sasl2/%{name}.conf

# Provided by package openbios
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/openbios-ppc
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/openbios-sparc32
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/openbios-sparc64
# Provided by package SLOF
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/slof.bin
# Provided by package ipxe-roms-qemu
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/pxe*rom
# Provided by package seavgabios
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/vgabios*bin
# Provided by package seabios-bin
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/bios.bin
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/bios-256k.bin
# Provided by package sgabios
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/sgabios.bin
# Provided by the OVMF and/or AAVMF packages
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/edk2*
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/firmware
# We don't package RISC-V
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/opensbi-riscv*
# We don't package UI elements
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/applications
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/icons
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/qemu-nsis.bmp
# We don't package the setuid root qemu-bridge-helper script
rm -rf ${RPM_BUILD_ROOT}%{_libexecdir}/qemu-bridge-helper
# We don't package elf2dmp
rm -rf ${RPM_BUILD_ROOT}%{_bindir}/elf2dmp

unused_blobs="QEMU,cgthree.bin QEMU,tcx.bin bamboo.dtb palcode-clipper \
petalogix-ml605.dtb petalogix-s3adsp1800.dtb ppc_rom.bin \
s390-ccw.img s390-zipl.rom spapr-rtas.bin u-boot.e500 \
skiboot.lid qemu_vga.ndrv s390-netboot.img canyonlands.dtb \
hppa-firmware.img u-boot-sam460-20100605.bin"
for blob in $unused_blobs; do
   rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/$blob
done

rm -rf ${RPM_BUILD_ROOT}%{_bindir}/qemu-edid
rm -rf ${RPM_BUILD_ROOT}%{_bindir}/qemu-keymap
rm -rf ${RPM_BUILD_ROOT}%{_bindir}/qemu-trace-stap

# Remove ivshmem example programs
rm -rf ${RPM_BUILD_ROOT}%{_bindir}/ivshmem-client
rm -rf ${RPM_BUILD_ROOT}%{_bindir}/ivshmem-server

# Remove ROM images unused by aarch64
%ifarch aarch64
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/efi-e1000.rom
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/efi-e1000e.rom
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/efi-eepro100.rom
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/efi-ne2k_pci.rom
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/efi-pcnet.rom
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/efi-rtl8139.rom
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/efi-virtio.rom
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/efi-vmxnet3.rom
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/kvmvapic.bin
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/linuxboot.bin
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/linuxboot_dma.bin
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/multiboot.bin
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/pvh.bin
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/bios-microvm.bin
%endif


pxe_link() {
	ln -s ../ipxe/$2.rom ${RPM_BUILD_ROOT}%{_datadir}/%{name}/pxe-$1.rom
}

%ifarch x86_64
pxe_link e1000 8086100e
pxe_link ne2k_pci 10ec8029
pxe_link pcnet 10222000
pxe_link rtl8139 10ec8139
pxe_link virtio 1af41000
pxe_link eepro100 80861209
pxe_link e1000e 808610d3
pxe_link vmxnet3 15ad07b0
%endif

rom_link() {
    ln -s $1 ${RPM_BUILD_ROOT}%{_datadir}/%{name}/$2
}

%ifarch x86_64
rom_link ../seavgabios/vgabios-isavga.bin vgabios.bin
rom_link ../seavgabios/vgabios-cirrus.bin vgabios-cirrus.bin
rom_link ../seavgabios/vgabios-qxl.bin vgabios-qxl.bin
rom_link ../seavgabios/vgabios-stdvga.bin vgabios-stdvga.bin
rom_link ../seavgabios/vgabios-vmware.bin vgabios-vmware.bin
rom_link ../seavgabios/vgabios-virtio.bin vgabios-virtio.bin
rom_link ../seabios/bios.bin bios.bin
rom_link ../seabios/bios-256k.bin bios-256k.bin
rom_link ../sgabios/sgabios.bin sgabios.bin
%endif

# Install rules to use the bridge helper with libvirt's virbr0
install -m 0644 %{_sourcedir}/bridge.conf $RPM_BUILD_ROOT%{_sysconfdir}/%{name}/bridge.conf

%if 0%{?have_qmpregdump}
# Install in /usr/bin
install -m 0755 %{_sourcedir}/qmp-regdump $RPM_BUILD_ROOT%{_bindir}/
# Install in /usr/lib/<python sitelib>/sos/report/plugins/
mkdir -p $RPM_BUILD_ROOT%{python3_sitelib}/sos/report/plugins
install -m 0755 %{_sourcedir}/qemu_regdump.py $RPM_BUILD_ROOT%{python3_sitelib}/sos/report/plugins
%endif

# We need to make the block device modules executable else
# RPM won't pick up their dependencies.
chmod +x $RPM_BUILD_ROOT%{_libdir}/qemu-kvm/block-*.so


%check


%post -n qemu-kvm-core
udevadm control --reload >/dev/null 2>&1 || :
udevadm trigger --subsystem-match=misc --sysname-match=kvm --action=add || :
chmod --quiet 666 /dev/kvm || :

%post -n qemu-kvm-common
getent group kvm >/dev/null || groupadd -g 36 -r kvm
getent group qemu >/dev/null || groupadd -g 107 -r qemu
getent passwd qemu >/dev/null || \
  useradd -r -u 107 -g qemu -G kvm -d / -s /sbin/nologin \
    -c "qemu user" qemu

%if 0%{?have_gluster}
%post block-gluster
mkdir -p /var/run/qemu/%{name}-%{version}-%{release}
cp %{_libdir}/qemu-kvm/block-gluster.so /var/run/qemu/%{name}-%{version}-%{release}/
%endif

%if 0%{?have_iscsi}
%post block-iscsi
mkdir -p /var/run/qemu/%{name}-%{version}-%{release}
cp %{_libdir}/qemu-kvm/block-iscsi.so /var/run/qemu/%{name}-%{version}-%{release}/
%endif

%if 0%{?have_rbd}
%post block-rbd
mkdir -p /var/run/qemu/%{name}-%{version}-%{release}
cp %{_libdir}/qemu-kvm/block-rbd.so /var/run/qemu/%{name}-%{version}-%{release}/
%endif

%if 0%{?have_ssh}
%post block-ssh
mkdir -p /var/run/qemu/%{name}-%{version}-%{release}
cp %{_libdir}/qemu-kvm/block-ssh.so /var/run/qemu/%{name}-%{version}-%{release}/
%endif

%if 0%{?have_curl}
%post block-curl
mkdir -p /var/run/qemu/%{name}-%{version}-%{release}
cp %{_libdir}/qemu-kvm/block-curl.so /var/run/qemu/%{name}-%{version}-%{release}/
%endif


%global qemu_kvm_files \
%{_libexecdir}/qemu-kvm \
%{_datadir}/systemtap/tapset/qemu-kvm.stp \
%{_datadir}/%{name}/trace-events-all \
%{_datadir}/systemtap/tapset/qemu-kvm-simpletrace.stp

%files
# Deliberately empty

%if 0%{?have_sdl_gtk}
%files qemu-kvm-common -f %{name}.lang
%else
%files -n qemu-kvm-common
%endif
%if 0%{?have_docs}
%dir %{qemudocdir}
%doc %{qemudocdir}/Changelog
%doc %{qemudocdir}/COPYING
%doc %{qemudocdir}/COPYING.LIB
%doc %{qemudocdir}/LICENSE
%doc %{qemudocdir}/qemu-doc.html
%doc %{qemudocdir}/qemu-doc.txt
%doc %{qemudocdir}/qemu-ga-ref.html
%doc %{qemudocdir}/qemu-ga-ref.txt
%doc %{qemudocdir}/qemu-qmp-ref.html
%doc %{qemudocdir}/qemu-qmp-ref.txt
%doc %{qemudocdir}/README
%{_mandir}/man1/qemu.1*
%{_mandir}/man1/virtfs-proxy-helper.1*
%{_mandir}/man7/qemu-ga-ref.7*
%{_mandir}/man7/qemu-qmp-ref.7*
%endif
%dir %{_datadir}/%{name}/
%{_datadir}/%{name}/keymaps/
%if 0%{?have_virtfs}
%{_bindir}/virtfs-proxy-helper
%endif
%{_bindir}/qemu-pr-helper
%dir %{_sysconfdir}/%{name}
%config(noreplace) %{_sysconfdir}/%{name}/bridge.conf
%ifarch x86_64
%config(noreplace) %{_sysconfdir}/modprobe.d/kvm.conf
%endif
%config(noreplace) %{_sysconfdir}/modprobe.d/vhost.conf
%config(noreplace) %{_sysconfdir}/sasl2/%{name}.conf
%{_datadir}/%{name}/simpletrace.py*
%{_datadir}/%{name}/tracetool/*.py*
%{_datadir}/%{name}/tracetool/backend/*.py*
%{_datadir}/%{name}/tracetool/format/*.py*
%if 0%{?have_qmpregdump}
%config(noreplace) %{_bindir}/qmp-regdump
%config(noreplace) %{python3_sitelib}/sos/report/plugins/qemu_regdump.py*
%config(noreplace) %{python3_sitelib}/sos/report/plugins/__pycache__/*.pyc
%endif

%files -n qemu-kvm-core
%defattr(-,root,root)
%ifarch x86_64
    %{_datadir}/%{name}/bios.bin
    %{_datadir}/%{name}/bios-256k.bin
    %{_datadir}/%{name}/linuxboot.bin
    %{_datadir}/%{name}/linuxboot_dma.bin
    %{_datadir}/%{name}/multiboot.bin
    %{_datadir}/%{name}/kvmvapic.bin
    %{_datadir}/%{name}/sgabios.bin
    %{_datadir}/%{name}/pvh.bin
    %{_datadir}/%{name}/bios-microvm.bin
    %{_datadir}/%{name}/vgabios.bin
    %{_datadir}/%{name}/vgabios-cirrus.bin
    %{_datadir}/%{name}/vgabios-qxl.bin
    %{_datadir}/%{name}/vgabios-stdvga.bin
    %{_datadir}/%{name}/vgabios-vmware.bin
    %{_datadir}/%{name}/vgabios-virtio.bin
    %{_datadir}/%{name}/pxe-e1000.rom
    %{_datadir}/%{name}/efi-e1000.rom
    %{_datadir}/%{name}/pxe-e1000e.rom
    %{_datadir}/%{name}/efi-e1000e.rom
    %{_datadir}/%{name}/pxe-eepro100.rom
    %{_datadir}/%{name}/efi-eepro100.rom
    %{_datadir}/%{name}/pxe-ne2k_pci.rom
    %{_datadir}/%{name}/efi-ne2k_pci.rom
    %{_datadir}/%{name}/pxe-pcnet.rom
    %{_datadir}/%{name}/efi-pcnet.rom
    %{_datadir}/%{name}/pxe-rtl8139.rom
    %{_datadir}/%{name}/efi-rtl8139.rom
    %{_datadir}/%{name}/pxe-virtio.rom
    %{_datadir}/%{name}/efi-virtio.rom
    %{_datadir}/%{name}/pxe-vmxnet3.rom
    %{_datadir}/%{name}/efi-vmxnet3.rom
%endif
%{_udevrulesdir}/80-kvm.rules
%{?qemu_kvm_files:}


%files -n qemu-img
%defattr(-,root,root)
%{_bindir}/qemu-img
%{_bindir}/qemu-io
%{_bindir}/qemu-nbd
%if 0%{?have_docs}
%{_mandir}/man1/qemu-img.1*
%{_mandir}/man8/qemu-nbd.8*
%endif


%if 0%{?have_agent}
%files -n qemu-guest-agent
%defattr(-,root,root,-)
%{_bindir}/qemu-ga
%if 0%{?have_docs}
%{_mandir}/man8/qemu-ga.8*
%endif
%{_unitdir}/qemu-guest-agent.service
%{_udevrulesdir}/99-qemu-guest-agent.rules
%config(noreplace) %{_sysconfdir}/sysconfig/qemu-ga
%{_sysconfdir}/qemu-ga
%{_datadir}/%{name}/qemu-ga
%dir %{_localstatedir}/log/qemu-ga
%endif


%if 0%{?have_gluster}
%files block-gluster
%{_libdir}/qemu-kvm/block-gluster.so
%endif


%if 0%{?have_iscsi}
%files block-iscsi
%{_libdir}/qemu-kvm/block-iscsi.so
%endif


%if 0%{?have_rbd}
%files block-rbd
%{_libdir}/qemu-kvm/block-rbd.so
%endif


%if 0%{?have_ssh}
%files block-ssh
%{_libdir}/qemu-kvm/block-ssh.so
%endif


%if 0%{?have_curl}
%files block-curl
%{_libdir}/qemu-kvm/block-curl.so
%endif


%changelog
* Fri Jun 16 2023 Karl Heubaum <karl.heubaum@oracle.com> - 4.2.1-27.el8
- hw/arm/virt: Add nvdimm hotplug support (Shameer Kolothum) [Orabug: 35505663]
- hw/arm/virt: Add nvdimm hot-plug infrastructure (Kwangwoo Lee) [Orabug: 35505663]
- nvdimm: Use configurable ACPI IO base and size (Kwangwoo Lee) [Orabug: 35505663]
- target/i386: define a new MSR based feature word - FEAT_PERF_CAPABILITIES (Like Xu) [Orabug: 35370615]

* Thu Apr 20 2023 Karl Heubaum <karl.heubaum@oracle.com> - 4.2.1-26.el8
- migration: check magic value for deciding the mapping of channels (Manish Mishra) [Orabug: 34735462]
- io: Add support for MSG_PEEK for socket channel (Manish Mishra) [Orabug: 34735462]
- migration: Move channel setup out of postcopy_try_recover() (Peter Xu) [Orabug: 34735462]
- vdpa: commit all host notifier MRs in a single MR transaction (Mike Longpeng) [Orabug: 35252234]
- vhost: configure all host notifiers in a single MR transaction (Mike Longpeng) [Orabug: 35252234]
- vhost: simplify vhost_dev_enable_notifiers (Mike Longpeng) [Orabug: 35252234]
- pcie: Do not update hotplugged device power in RUN_STATE_INMIGRATE state (Annie Li) [Orabug: 35055290]
- qga/win32: Use rundll for VSS installation (Konstantin Kostiuk) [Orabug: 35206108] {CVE-2023-0664}
- qga/win32: Remove change action from MSI installer (Konstantin Kostiuk) [Orabug: 35206108] {CVE-2023-0664}
- hw/display/qxl: Assert memory slot fits in preallocated MemoryRegion (Philippe Mathieu-Daudé) [Orabug: 34846087]
- hw/display/qxl: Avoid buffer overrun in qxl_phys2virt (CVE-2022-4144) (Philippe Mathieu-Daudé) [Orabug: 34846087] {CVE-2022-4144}
- hw/display/qxl: Pass requested buffer size to qxl_phys2virt() (Philippe Mathieu-Daudé) [Orabug: 34846087]
- hw/display/qxl: Document qxl_phys2virt() (Philippe Mathieu-Daudé) [Orabug: 34846087]
- hw/display/qxl: Have qxl_log_command Return early if no log_cmd handler (Philippe Mathieu-Daudé) [Orabug: 34846087]
- virtio-blk: On restart, process queued requests in the proper context (Sergio Lopez) [Orabug: 35060530]
- virtio-blk: Refactor the code that processes queued requests (Sergio Lopez) [Orabug: 35060530]
- hw/intc/ioapic: Update KVM routes before redelivering IRQ, on RTE update (David Woodhouse) [Orabug: 35219223]
- modules: load modules from /var/run/qemu/ directory firstly (Siddhi Katage) [Orabug: 34867783]
- qemu.spec: Add post-install script for block storage modules (Siddhi Katage) [Orabug: 34867783]
- qemu.spec: Enable '-module-upgrades' for OL7 (Siddhi Katage) [Orabug: 34867783]
- module: increase dirs array size by one (Bruce Rogers) [Orabug: 34867783]
- modules: load modules from versioned /var/run dir (Christian Ehrhardt) [Orabug: 34867783]
- blockjob: Fix crash with IOthread when block commit after snapshot (Michael Qiu) [Orabug: 35118668]

* Fri Feb 10 2023 Karl Heubaum <karl.heubaum@oracle.com> - 4.2.1-25.el8
- target/i386/kvm: get and put AMD pmu registers (Dongli Zhang) [Orabug: 34641255]
- i386/kvm: fix a use-after-free when vcpu plug/unplug (Pan Nengyuan) [Orabug: 34859902]
- memory: batch allocate ioeventfds[] in address_space_update_ioeventfds() (Stefan Hajnoczi) [Orabug: 34538900]
- virtio-blk: Fix clean up of host notifiers for single MR transaction (Mark Mielke) [Orabug: 34538900]
- virtio-scsi: Configure all host notifiers in a single MR transaction (Greg Kurz) [Orabug: 34538900]
- virtio-scsi: Set host notifiers and callbacks separately (Greg Kurz) [Orabug: 34538900]
- virtio-blk: Configure all host notifiers in a single MR transaction (Greg Kurz) [Orabug: 34538900]
- virtio-blk: Fix rollback path in virtio_blk_data_plane_start() (Greg Kurz) [Orabug: 34538900]
- event_notifier: Set ->initialized earlier in event_notifier_init() (Greg Kurz) [Orabug: 34538900]
- virtio-scsi: don't process IO on fenced dataplane (Maxim Levitsky) [Orabug: 34538900]
- virtio-scsi: don't uninitialize queues that we didn't initialize (Maxim Levitsky) [Orabug: 34538900]
- hw/arm/virt: build SMBIOS 19 table (Mihai Carabas)

* Thu Jan 5 2023 Karl Heubaum <karl.heubaum@oracle.com> - 4.2.1-24.el8
- hw/display/ati_2d: Fix buffer overflow in ati_2d_blt (CVE-2021-3638) (Philippe Mathieu-Daudé) [Orabug: 33930374] {CVE-2021-3638}
- tests/acpi: virt: update ACPI MADT and FADT binaries (Miguel Luis)
- acpi: arm/virt: madt: bump to revision 4 accordingly to ACPI 6.0 Errata A (Miguel Luis)
- acpi: arm/virt: madt: use build_append_int_noprefix() API to compose MADT table (Igor Mammedov)
- acpi: madt: arm/x86: use acpi_table_begin()/acpi_table_end() instead of build_header() (Igor Mammedov)
- hw/arm/virt-acpi-build:Remove dead assignment in build_madt() (Chen Qun)
- acpi: build_fadt: adapt FADT table names (Miguel Luis)
- acpi: fadt: support revision 6.0 of the ACPI specification (Miguel Luis)
- tests/acpi: virt: allow acpi MADT and FADT changes (Miguel Luis)
- Document CVE-2022-3165 as not applicable (Mark Kanda) [Orabug: 34713999] {CVE-2022-3165}
- Document CVE-2022-1050 as not applicable (Mark Kanda) [Orabug: 34132133] {CVE-2022-1050}
- hw/acpi/erst.c: Fix memory handling issues (Christian A. Ehrhardt) [Orabug: 34779472] {CVE-2022-4172}
- vhost-vdpa: fix assert !virtio_net_get_subqueue(nc)->async_tx.elem in virtio_net_reset (Si-Wei Liu)
- net/vhost-vdpa.c: Fix clang compilation failure (Peter Maydell)
- vhost-vdpa: allow passing opened vhostfd to vhost-vdpa (Si-Wei Liu)
- hw/acpi/aml-build: Improve scalability of PPTT generation (Yanan Wang)
- tests/data/acpi/virt: update empty file for PPTT (Miguel Luis)
- hw/arm/virt-acpi-build: Generate PPTT table (Yanan Wang)
- tests/data/acpi/virt: Add an empty expected file for PPTT (Yanan Wang)
- hw/acpi/aml-build: Add PPTT table (Andrew Jones)
- hw/acpi/aml-build: Add Processor hierarchy node structure (Yanan Wang)
- machine: Add SMP Sockets in CpuTopology (Babu Moger)
- bios-tables-test: generate table for virt/DBG2 (Miguel Luis)
- hw/arm/virt_acpi_build: Generate DBG2 table (Eric Auger)
- tests/acpi: Add void table for virt/DBG2 bios-tables-test (Eric Auger)
- tests/acpi: virt: update ACPI GTDT binaries (Miguel Luis) [Orabug: 34711916]
- acpi: arm/virt: build_gtdt: fix invalid 64-bit physical addresses (Miguel Luis) [Orabug: 34711916]
- tests/acpi: virt: allow acpi GTDT changes (Miguel Luis) [Orabug: 34711916]
- acpi: fix OEM ID/OEM Table ID padding (Igor Mammedov) [Orabug: 34711916]
- acpi: arm/virt: build_gtdt: use acpi_table_begin()/acpi_table_end() instead of build_header() (Igor Mammedov) [Orabug: 34711916]
- acpi: add helper routines to initialize ACPI tables (Igor Mammedov) [Orabug: 34711916]
- acpi: declare the default assignable value for the ACPI table header (Miguel Luis) [Orabug: 34711916]

* Tue Nov 29 2022 Karl Heubaum <karl.heubaum@oracle.com> - 4.2.1-22.el8
- Revert "block: Set the name of BlockBackend if possible" (Joe Jin) [Orabug: 34841102]
- Revert "iotests: Adjust 186.out to account for 'null' node-name" (Joe Jin) [Orabug: 34841102]

* Mon Oct 10 2022 Karl Heubaum <karl.heubaum@oracle.com> - 4.2.1-21.el8
- qemu-kvm.spec: Fix the qemu-regdump sos report plugin path (Mark Kanda) [Orabug: 34680062]
- qmp-regdump: Require python3 on OL8 (Mark Kanda) [Orabug: 34672256]
- iotests: Adjust 186.out to account for 'null' node-name (Mark Kanda) [Orabug: 34447388]
- block: Set the name of BlockBackend if possible (Annie Li) [Orabug: 34447388]
- acpi: Update _DSM method in expected files (Mark Kanda) [Orabug: 34616322]
- acpi/gpex: Fix cca attribute check for pxb device (Xingang Wang) [Orabug: 34616322]
- acpi: Enable pxb unit-test for ARM virt machine (Jiahui Cen) [Orabug: 34616322]
- Kconfig: Compile PXB for ARM_VIRT (Jiahui Cen) [Orabug: 34616322]
- acpi/gpex: Exclude pxb's resources from PCI0 (Jiahui Cen) [Orabug: 34616322]
- acpi/gpex: Inform os to keep firmware resource map (Jiahui Cen) [Orabug: 34616322]
- acpi: Add addr offset in build_crs (Jiahui Cen) [Orabug: 34616322]
- unit-test: Add testcase for pxb (Yubo Miao) [Orabug: 34616322]
- acpi: Align the size to 128k (Yubo Miao) [Orabug: 34616322]
- acpi/gpex: Build tables for pxb (Yubo Miao) [Orabug: 34616322]
- acpi: Extract crs build form acpi_build.c (Yubo Miao) [Orabug: 34616322]
- hw/arm/virt: Write extra pci roots into fw_cfg (Jiahui Cen) [Orabug: 34616322]
- fw_cfg: Refactor extra pci roots addition (Jiahui Cen) [Orabug: 34616322]
- acpi/gpex: Extract two APIs from acpi_dsdt_add_pci (Yubo Miao) [Orabug: 34616322]
- arm: use acpi_dsdt_add_gpex (Gerd Hoffman) [Orabug: 34616322]
- acpi: add acpi_dsdt_add_gpex (Gerd Hoffman) [Orabug: 34616322]
- acpi: Allow DSDT acpi table changes (Jiahui Cen) [Orabug: 34616322]
- move MemMapEntry (Gerd Hoffman) [Orabug: 34616322]
- scsi/lsi53c895a: really fix use-after-free in lsi_do_msgout (CVE-2022-0216) (Mauro Matteo Cascella) [Orabug: 34353672] {CVE-2022-0216}
- scsi/lsi53c895a: fix use-after-free in lsi_do_msgout (CVE-2022-0216) (Mauro Matteo Cascella) [Orabug: 34353672] {CVE-2022-0216}
- tests/qtest: Add fuzz-lsi53c895a-test (Philippe Mathieu-Daudé) [Orabug: 34353672] {CVE-2022-0216}
- hw/scsi/lsi53c895a: Do not abort when DMA requested and no data queued (Philippe Mathieu-Daudé) [Orabug: 34353672] {CVE-2022-0216}
- vfio: defer to commit kvm irq routing when enable msi/msix (Mike Longpeng) [Orabug: 34419422]
- vfio: simplify the failure path in vfio_msi_enable (Mike Longpeng) [Orabug: 34419422]
- vfio: move re-enabling INTX out of the common helper (Mike Longpeng) [Orabug: 34419422]
- vfio: simplify the conditional statements in vfio_msi_enable (Mike Longpeng) [Orabug: 34419422]
- kvm/msi: do explicit commit when adding msi routes (Mike Longpeng) [Orabug: 34419422]
- kvm-irqchip: introduce new API to support route change (Mike Longpeng) [Orabug: 34419422]
- event_notifier: handle initialization failure better (Maxim Levitsky) [Orabug: 34419422]
- qmp-regdump: use QMP command 'query-cpus-fast' (Mark Kanda) [Orabug: 34510460]

* Wed Aug 31 2022 Karl Heubaum <karl.heubaum@oracle.com> - 4.2.1-20.el8
- vl: Add an -action option to override MCE handling (Mark Kanda) [Orabug: 34538907]
- vl: Add an -action option specifying response to guest events (Alejandro Jimenez) [Orabug: 34538907]
- qmp: generalize watchdog-set-action to -no-reboot/-no-shutdown (Alejandro Jimenez) [Orabug: 34538907]

* Thu Aug 18 2022 Karl Heubaum <karl.heubaum@oracle.com> - 4.2.1-19.el8
- virtio-net: handle zero mac for a vdpa peer (Cindy Lu) [Orabug: 34137522]
- virtio-net: don't handle mq request in userspace handler for vhost-vdpa (Si-Wei Liu) [Orabug: 34137522]
- vhost-vdpa: change name and polarity for vhost_vdpa_one_time_request() (Si-Wei Liu) [Orabug: 34137522]
- vhost-vdpa: backend feature should set only once (Si-Wei Liu) [Orabug: 34137522]
- vhost-net: fix improper cleanup in vhost_net_start (Si-Wei Liu) [Orabug: 34137522]
- vhost-vdpa: fix improper cleanup in net_init_vhost_vdpa (Si-Wei Liu) [Orabug: 34137522]
- virtio-net: align ctrl_vq index for non-mq guest for vhost_vdpa (Si-Wei Liu) [Orabug: 34137522]
- virtio-net: setup vhost_dev and notifiers for cvq only when feature is negotiated (Si-Wei Liu) [Orabug: 34137522]
- virtio: fix the condition for iommu_platform not supported (Halil Pasic) [Orabug: 34137522]
- vdpa: Make ncs autofree (Eugenio Pérez) [Orabug: 34137522]
- vhost-vdpa: make notifiers _init()/_uninit() symmetric (Laurent Vivier) [Orabug: 34137522]
- hw/virtio: vdpa: Fix leak of host-notifier memory-region (Laurent Vivier) [Orabug: 34137522]
- vhost-vdpa: stick to -errno error return convention (Roman Kagan) [Orabug: 34137522]
- vdpa: Add dummy receive callback (Eugenio Pérez) [Orabug: 34137522]
- vdpa: Check for existence of opts.vhostdev (Eugenio Pérez) [Orabug: 34137522]
- vhost: Fix last vq queue index of devices with no cvq (Eugenio Pérez) [Orabug: 34137522]
- vhost: Rename last_index to vq_index_end (Eugenio Pérez) [Orabug: 34137522]
- net/vhost-vdpa: fix memory leak in vhost_vdpa_get_max_queue_pairs() (Stefano Garzarella) [Orabug: 34137522]
- vhost-vdpa: multiqueue support (Jason Wang) [Orabug: 34137522]
- virtio-net: vhost control virtqueue support (Jason Wang) [Orabug: 34137522]
- vhost: record the last virtqueue index for the virtio device (Jason Wang) [Orabug: 34137522]
- virtio-net: use "queue_pairs" instead of "queues" when possible (Jason Wang) [Orabug: 34137522]
- vhost-net: control virtqueue support (Jason Wang) [Orabug: 34137522]
- net: introduce control client (Jason Wang) [Orabug: 34137522]
- vhost-vdpa: let net_vhost_vdpa_init() returns NetClientState * (Jason Wang) [Orabug: 34137522]
- vhost-vdpa: prepare for the multiqueue support (Jason Wang) [Orabug: 34137522]
- vhost-vdpa: classify one time request (Jason Wang) [Orabug: 34137522]
- vhost-vdpa: open device fd in net_init_vhost_vdpa() (Jason Wang) [Orabug: 34137522]
- vdpa: Check for iova range at mappings changes (Eugenio Pérez) [Orabug: 34137522]
- vdpa: Add vhost_vdpa_section_end (Eugenio Pérez) [Orabug: 34137522]
- net/vhost-vdpa: Fix device compatibility check (Kevin Wolf) [Orabug: 34137522]
- net/vhost-user: Fix device compatibility check (Kevin Wolf) [Orabug: 34137522]
- net: Introduce NetClientInfo.check_peer_type() (Kevin Wolf) [Orabug: 34137522]
- vhost-vdpa: remove the unncessary queue_index assignment (Jason Wang) [Orabug: 34137522]
- vhost-vdpa: fix the wrong assertion in vhost_vdpa_init() (Jason Wang) [Orabug: 34137522]
- vhost-vdpa: tweak the error label in vhost_vdpa_add() (Jason Wang) [Orabug: 34137522]
- vhost-vdpa: fix leaking of vhost_net in vhost_vdpa_add() (Jason Wang) [Orabug: 34137522]
- vhost-vdpa: don't cleanup twice in vhost_vdpa_add() (Jason Wang) [Orabug: 34137522]
- vhost-vdpa: remove the unnecessary check in vhost_vdpa_add() (Jason Wang) [Orabug: 34137522]
- vhost_net: do not assume nvqs is always 2 (Jason Wang) [Orabug: 34137522]
- vhost: use unsigned int for nvqs (Jason Wang) [Orabug: 34137522]
- vhost_net: remove the meaningless assignment in vhost_net_start_one() (Jason Wang) [Orabug: 34137522]
- vhost-vdpa: correctly return err in vhost_vdpa_set_backend_cap() (Jason Wang) [Orabug: 34137522]
- vhost-vdpa: remove unused variable "acked_features" (Jason Wang) [Orabug: 34137522]
- vhost: correctly detect the enabling IOMMU (Jason Wang) [Orabug: 34137522]
- virtio-pci: implement iommu_enabled() (Jason Wang) [Orabug: 34137522]
- virtio-bus: introduce iommu_enabled() (Jason Wang) [Orabug: 34137522]
- hw/virtio: Fix leak of host-notifier memory-region (Yajun Wu) [Orabug: 34137522]
- vhost-vdpa: Do not send empty IOTLB update batches (Eugenio Pérez) [Orabug: 34137522]
- vhost-vdpa: remove the unused vhost_vdpa_get_acked_features() (Jason Wang) [Orabug: 34137522]
- vhost-vdpa: don't initialize backend_features (Jason Wang) [Orabug: 34137522]
- vhost-vdpa: map virtqueue notification area if possible (Jason Wang) [Orabug: 34137522]
- vhost-vdpa: skip ram device from the IOTLB mapping (Jason Wang) [Orabug: 34137522]
- vhost-vdpa: Remove redundant declaration of address_space_memory (Xie Yongji) [Orabug: 34137522]
- virtio: Fail if iommu_platform is requested, but unsupported (Kevin Wolf) [Orabug: 34137522]
- vhost-vdpa: Make vhost_vdpa_get_device_id() static (Zenghui Yu) [Orabug: 34137522]
- Update linux headers to 5.11-rc2 (Eric Farman) [Orabug: 34137522]
- linux-headers: update against 5.10-rc1 (Matthew Rosato) [Orabug: 34137522]
- linux headers: sync to 5.9-rc7 (Andrew Jones) [Orabug: 34137522]

* Thu Jun 30 2022 Karl Heubaum <karl.heubaum@oracle.com> - 4.2.1-18.el8
- block: introduce max_hw_iov for use in scsi-generic (Paolo Bonzini) [Orabug: 33785156]
- file-posix: try BLKSECTGET on block devices too, do not round to power of 2 (Paolo Bonzini) [Orabug: 33785156]
- block: add max_hw_transfer to BlockLimits (Paolo Bonzini) [Orabug: 33785156]
- block-backend: align max_transfer to request alignment (Paolo Bonzini) [Orabug: 33785156]
- osdep: provide ROUND_DOWN macro (Paolo Bonzini) [Orabug: 33785156]
- scsi-generic: pass max_segments via max_iov field in BlockLimits (Paolo Bonzini) [Orabug: 33785156]
- file-posix: fix max_iov for /dev/sg devices (Paolo Bonzini) [Orabug: 33785156]
- display/qxl-render: fix race condition in qxl_cursor (CVE-2021-4207) (Mauro Matteo Cascella) [Orabug: 34049511] {CVE-2021-4207}
- ui/cursor: fix integer overflow in cursor_alloc (CVE-2021-4206) (Mauro Matteo Cascella) [Orabug: 34049509] {CVE-2021-4206}
- hw/block/fdc: Prevent end-of-track overrun (CVE-2021-3507) (Philippe Mathieu-Daudé) [Orabug: 32860387] {CVE-2021-3507}
- pc: q35: Bump max_cpus to 512 (Suravee Suthikulpanit) [Orabug: 34314249]
- tests/qtest: fix pvpanic-pci-test (Mark Kanda) [Orabug: 34284763]
- libqos: pci-pc: use 32-bit write for EJ register (Paolo Bonzini) [Orabug: 34284758]
- libqos: usb-hcd-ehci: use 32-bit write for config register (Paolo Bonzini) [Orabug: 34284768]
- target/i386/kvm: Fix disabling MPX on "-cpu host" with MPX-capable host (Maciej S. Szmigiero) [Orabug: 33528615]
- i386: Mask SVM features if nested SVM is disabled (Eduardo Habkost) [Orabug: 33860224]
- ide: Cap LBA28 capacity announcement to 2^28-1 (Samuel Thibault) [Orabug: 25327652]
- tests/acpi: update expected arm/virt tables (Mark Kanda) [Orabug: 34132842]

* Tue May 3 2022 Karl Heubaum <karl.heubaum@oracle.com> - 4.2.1-17.el8
- arm/acpi: fix an out of spec _UID for PCI root (Michael S. Tsirkin)
- arm/acpi: fix duplicated _UID of PCI interrupt link devices (Heyi Guo)
- arm/acpi: fix PCI _PRT definition (Heyi Guo)
- docs: fix references to docs/devel/atomics.rst (Stefano Garzarella) [Orabug: 33659123]
- rcu: do not mention atomic_mb_read/set in documentation (Paolo Bonzini) [Orabug: 33659123]
- atomics: update documentation (Paolo Bonzini) [Orabug: 33659123]
- atomics: convert to reStructuredText (Paolo Bonzini) [Orabug: 33659123]
- async: use explicit memory barriers (Paolo Bonzini) [Orabug: 33659123]
- aio-wait: delegate polling of main AioContext if BQL not held (Paolo Bonzini) [Orabug: 33659123]
- qapi: Add '@allow-write-only-overlay' feature for 'blockdev-snapshot' (Peter Krempa) [Orabug: 33888021]
- iotests: Add iothread cases to 155 (Kevin Wolf) [Orabug: 33888021]
- block: Fix cross-AioContext blockdev-snapshot (Kevin Wolf) [Orabug: 33888021]
- iotests: Test mirror with temporarily disabled target backing file (Kevin Wolf) [Orabug: 33888021]
- iotests: Fix run_job() with use_log=False (Kevin Wolf) [Orabug: 33888021]
- block: Relax restrictions for blockdev-snapshot (Kevin Wolf) [Orabug: 33888021]
- block: Make bdrv_get_cumulative_perm() public (Kevin Wolf) [Orabug: 33888021]
- iotests: Use complete_and_wait() in 155 (Max Reitz) [Orabug: 33888021]
- iotests: Support job-complete in run_job() (Kevin Wolf) [Orabug: 33888021]
- linux-headers: update again to 5.8 (Paolo Bonzini) [Orabug: 34022218]
- virtio-net: fix map leaking on error during receive (Jason Wang) [Orabug: 33941879] {CVE-2022-26353}
- vhost-vsock: detach the virqueue element in case of error (Stefano Garzarella) [Orabug: 33941844] {CVE-2022-26354}
- virtio-net: fix use after unmap/free for sg (Jason Wang) [Orabug: 33972912] {CVE-2021-3748}
- migration: Report the error returned when save_live_iterate fails (David Edmondson)

* Tue Mar 8 2022 Karl Heubaum <karl.heubaum@oracle.com> - 4.2.1-16.el8
- Document CVE-2021-4145 as fixed (Mark Kanda) [Orabug: 33791496] {CVE-2021-4145}
- migration: Tally pre-copy, downtime and post-copy bytes independently (David Edmondson)
- migration: Introduce ram_transferred_add() (David Edmondson)
- ACPI ERST: specification for ERST support (Eric DeVolder)
- ACPI ERST: step 6 of bios-tables-test.c (Eric DeVolder)
- ACPI ERST: bios-tables-test testcase (Eric DeVolder)
- ACPI ERST: qtest for ERST (Eric DeVolder)
- ACPI ERST: create ACPI ERST table for pc/x86 machines (Eric DeVolder)
- ACPI ERST: build the ACPI ERST table (Eric DeVolder)
- ACPI ERST: support for ACPI ERST feature (Eric DeVolder)
- ACPI ERST: header file for ERST (Eric DeVolder)
- ACPI ERST: PCI device_id for ERST (Eric DeVolder)
- ACPI ERST: bios-tables-test.c steps 1 and 2 (Eric DeVolder)
- ACPI: cleanup bios-tables-test state (Eric DeVolder)
- KVM: x86: believe what KVM says about WAITPKG (Paolo Bonzini) [Orabug: 33832295]
- cputlb: destroy CPUTLB with tlb_destroy (Emilio G. Cota) [Orabug: 33428107]

* Wed Jan 19 2022 Karl Heubaum <karl.heubaum@oracle.com> - 4.2.1-15.el8
- qemu-kvm.spec: Add support for reading vmdk, vhdx, vpc, https, and ssh disk image formats from qemu-kvm (Karl Heubaum) [Orabug: 33741340]
- Document CVE-2021-4158 and CVE-2021-3947 as fixed (Mark Kanda) [Orabug: 33719302] [Orabug: 33754145] {CVE-2021-4158} {CVE-2021-3947}
- hw/block/fdc: Kludge missing floppy drive to fix CVE-2021-20196 (Philippe Mathieu-Daudé) [Orabug: 32439466] {CVE-2021-20196}
- hw/block/fdc: Extract blk_create_empty_drive() (Philippe Mathieu-Daudé) [Orabug: 32439466] {CVE-2021-20196}
- net: vmxnet3: validate configuration values during activate (CVE-2021-20203) (Prasad J Pandit) [Orabug: 32559476] {CVE-2021-20203}
- lan9118: switch to use qemu_receive_packet() for loopback (Alexander Bulekov) [Orabug: 32560540] {CVE-2021-3416}
- pcnet: switch to use qemu_receive_packet() for loopback (Alexander Bulekov) [Orabug: 32560540] {CVE-2021-3416}
- rtl8139: switch to use qemu_receive_packet() for loopback (Alexander Bulekov) [Orabug: 32560540] {CVE-2021-3416}
- tx_pkt: switch to use qemu_receive_packet_iov() for loopback (Jason Wang) [Orabug: 32560540] {CVE-2021-3416}
- sungem: switch to use qemu_receive_packet() for loopback (Jason Wang) [Orabug: 32560540] {CVE-2021-3416}
- dp8393x: switch to use qemu_receive_packet() for loopback packet (Jason Wang) [Orabug: 32560540] {CVE-2021-3416}
- e1000: switch to use qemu_receive_packet() for loopback (Jason Wang) [Orabug: 32560540] {CVE-2021-3416}
- net: introduce qemu_receive_packet() (Jason Wang) [Orabug: 32560540] {CVE-2021-3416}
- target/i386: Populate x86_ext_save_areas offsets using cpuid where possible (Paolo Bonzini)
- target/i386: Observe XSAVE state area offsets (Paolo Bonzini)
- target/i386: Make x86_ext_save_areas visible outside cpu.c (Paolo Bonzini)
- target/i386: Pass buffer and length to XSAVE helper (Paolo Bonzini)
- target/i386: Clarify the padding requirements of X86XSaveArea (Paolo Bonzini)
- target/i386: Consolidate the X86XSaveArea offset checks (Paolo Bonzini)
- target/i386: Declare constants for XSAVE offsets (Paolo Bonzini)

* Wed Dec 22 2021 Karl Heubaum <karl.heubaum@oracle.com> - 4.2.1-14.el8
- scsi: fix sense code for EREMOTEIO (Paolo Bonzini) [Orabug: 33537443]
- scsi: move host_status handling into SCSI drivers (Hannes Reinecke) [Orabug: 33537443]
- scsi: inline sg_io_sense_from_errno() into the callers (Hannes Reinecke) [Orabug: 33537443]
- scsi-generic: do not snoop the output of failed commands (Paolo Bonzini) [Orabug: 33537443]
- scsi: Add mapping for generic SCSI_HOST status to sense codes (Hannes Reinecke) [Orabug: 33537443]
- scsi: Rename linux-specific SG_ERR codes to generic SCSI_HOST error codes (Hannes Reinecke) [Orabug: 33537443]
- scsi: drop 'result' argument from command_complete callback (Hannes Reinecke) [Orabug: 33537443]
- scsi-disk: pass guest recoverable errors through even for rerror=stop (Paolo Bonzini) [Orabug: 33537443]
- scsi-disk: pass SCSI status to scsi_handle_rw_error (Paolo Bonzini) [Orabug: 33537443]
- scsi: introduce scsi_sense_from_errno() (Paolo Bonzini) [Orabug: 33537443]
- scsi-disk: do not complete requests early for rerror/werror=ignore (Paolo Bonzini) [Orabug: 33537443]
- scsi-disk: move scsi_handle_rw_error earlier (Paolo Bonzini) [Orabug: 33537443]
- scsi-disk: convert more errno values back to SCSI statuses (Paolo Bonzini) [Orabug: 33537443]

* Wed Dec 15 2021 Karl Heubaum <karl.heubaum@oracle.com> - 4.2.1-13.el8
- pcie: Do not set power state for some hot-plugged devices (Annie Li) [Orabug: 33642532]

* Thu Dec 2 2021 Karl Heubaum <karl.heubaum@oracle.com> - 4.2.1-12.1.el8
- Update slirp to address various CVEs (Mark Kanda) [Orabug: 32208456] [Orabug: 33014409] [Orabug: 33014414] [Orabug: 33014417] [Orabug: 33014420] {CVE-2020-29129} {CVE-2020-29130} {CVE-2021-3592} {CVE-2021-3593} {CVE-2021-3594} {CVE-2021-3595}
- hw/pflash_cfi01: Allow backing devices to be smaller than memory region (David Edmondson)
- pcie: expire pending delete (Gerd Hoffmann) [Orabug: 33450706]
- pcie: fast unplug when slot power is off (Gerd Hoffmann) [Orabug: 33450706]
- pcie: factor out pcie_cap_slot_unplug() (Gerd Hoffmann) [Orabug: 33450706]
- pcie: add power indicator blink check (Gerd Hoffmann) [Orabug: 33450706]
- pcie: implement slot power control for pcie root ports (Gerd Hoffmann) [Orabug: 33450706]
- pci: implement power state (Gerd Hoffmann) [Orabug: 33450706]
- hw/pci/pcie: Move hot plug capability check to pre_plug callback (Julia Suvorova) [Orabug: 33450706]
- hw/pci/pcie: Replace PCI_DEVICE() casts with existing variable (Julia Suvorova) [Orabug: 33450706]
- hw/pci/pcie: Forbid hot-plug if it's disabled on the slot (Julia Suvorova) [Orabug: 33450706]
- pcie_root_port: Add hotplug disabling option (Julia Suvorova) [Orabug: 33450706]
- qdev-monitor: Forbid repeated device_del (Julia Suvorova) [Orabug: 33450706]
- i386:acpi: Remove _HID from the SMBus ACPI entry (Corey Minyard)
- uas: add stream number sanity checks (Gerd Hoffmann) [Orabug: 33280793] {CVE-2021-3713}
- usbredir: fix free call (Gerd Hoffmann) [Orabug: 33198441] {CVE-2021-3682}
- hw/scsi/scsi-disk: MODE_PAGE_ALLS not allowed in MODE SELECT commands (Mauro Matteo Cascella) [Orabug: 33548490] {CVE-2021-3930}
- e1000: fix tx re-entrancy problem (Jon Maloy) [Orabug: 32560552] {CVE-2021-20257}
- virtio-net-pci: Don't use "efi-virtio.rom" on AArch64 (Mark Kanda) [Orabug: 33537594]
- MAINTAINERS: Add ACPI/HEST/GHES entries (Dongjiu Geng)
- target-arm: kvm64: handle SIGBUS signal from kernel or KVM (Dongjiu Geng)
- ACPI: Record Generic Error Status Block(GESB) table (Dongjiu Geng)
- KVM: Move hwpoison page related functions into kvm-all.c (Dongjiu Geng)
- ACPI: Record the Generic Error Status Block address (Dongjiu Geng)
- ACPI: Build Hardware Error Source Table (Dongjiu Geng)
- ACPI: Build related register address fields via hardware error fw_cfg blob (Dongjiu Geng)
- docs: APEI GHES generation and CPER record description (Dongjiu Geng)
- hw/arm/virt: Introduce a RAS machine option (Dongjiu Geng)
- acpi: nvdimm: change NVDIMM_UUID_LE to a common macro (Dongjiu Geng)
- block/curl: HTTP header field names are case insensitive (David Edmondson) [Orabug: 33287589]
- block/curl: HTTP header fields allow whitespace around values (David Edmondson) [Orabug: 33287589]

* Sun Sep 12 2021 Karl Heubaum <karl.heubaum@oracle.com> - 4.2.1-11.el8
- trace: use STAP_SDT_V2 to work around symbol visibility (Stefan Hajnoczi) [Orabug: 33272428]

* Tue Jul 27 2021 Karl Heubaum <karl.heubaum@oracle.com> - 4.2.1-11.el8
- pvrdma: Fix the ring init error flow (Marcel Apfelbaum) [Orabug: 33120142] {CVE-2021-3608}
- pvrdma: Ensure correct input on ring init (Marcel Apfelbaum) [Orabug: 33120146] {CVE-2021-3607}
- hw/rdma: Fix possible mremap overflow in the pvrdma device (Marcel Apfelbaum) [Orabug: 33120084] {CVE-2021-3582}
- vhost-user-gpu: reorder free calls (Gerd Hoffmann) [Orabug: 32950701] {CVE-2021-3544}
- vhost-user-gpu: abstract vg_cleanup_mapping_iov (Li Qiang) [Orabug: 32950716] {CVE-2021-3546}
- vhost-user-gpu: fix OOB write in 'virgl_cmd_get_capset' (Li Qiang) [Orabug: 32950716] {CVE-2021-3546}
- vhost-user-gpu: fix memory leak in 'virgl_resource_attach_backing' (Li Qiang) [Orabug: 32950701] {CVE-2021-3544}
- vhost-user-gpu: fix memory leak in 'virgl_cmd_resource_unref' (Li Qiang) [Orabug: 32950701] {CVE-2021-3544}
- vhost-user-gpu: fix memory leak while calling 'vg_resource_unref' (Li Qiang) [Orabug: 32950701] {CVE-2021-3544}
- vhost-user-gpu: fix memory leak in vg_resource_attach_backing (Li Qiang) [Orabug: 32950701] {CVE-2021-3544}
- vhost-user-gpu: fix resource leak in 'vg_resource_create_2d' (Li Qiang) [Orabug: 32950701] {CVE-2021-3544}
- vhost-user-gpu: fix memory disclosure in virgl_cmd_get_capset_info (Li Qiang) [Orabug: 32950708] {CVE-2021-3545}
- usb: limit combined packets to 1 MiB (Gerd Hoffmann) [Orabug: 32842778] {CVE-2021-3527}
- usb/redir: avoid dynamic stack allocation (Gerd Hoffmann) [Orabug: 32842778] {CVE-2021-3527}
- mptsas: Remove unused MPTSASState 'pending' field (Michael Tokarev) [Orabug: 32470463] {CVE-2021-3392}
- oslib-posix: initialize backend memory objects in parallel (Mark Kanda) [Orabug: 32555402]
- oslib-posix: refactor memory prealloc threads (Mark Kanda) [Orabug: 32555402]

* Tue Jun 29 2021 Karl Heubaum <karl.heubaum@oracle.com> - 4.2.1-10.el8
- e1000: fail early for evil descriptor (Jason Wang) [Orabug: 32560552] {CVE-2021-20257}
- Document CVE-2020-27661 as fixed (Mark Kanda) [Orabug: 32960200] {CVE-2020-27661}
- block: Avoid stale pointer dereference in blk_get_aio_context() (Greg Kurz)
- block: Fix blk->in_flight during blk_wait_while_drained() (Kevin Wolf)
- block: Increase BB.in_flight for coroutine and sync interfaces (Kevin Wolf)
- block-backend: Reorder flush/pdiscard function definitions (Kevin Wolf)
- i386/pc: let iterator handle regions below 4G (Joao Martins)
- arm/virt: Add memory hot remove support (Shameer Kolothum) [Orabug: 32643506]
- i386/pc: consolidate usable iova iteration (Joao Martins)
- i386/acpi: fix SRAT ranges in accordance to usable IOVA (Joao Martins)
- migration: increase listening socket backlog (Elena Ufimtseva)
- multifd: Make multifd_save_setup() get an Error parameter (Juan Quintela)
- multifd: Make multifd_load_setup() get an Error parameter (Juan Quintela)
- migration: fix maybe-uninitialized warning (Marc-André Lureau)
- migration: Fix the re-run check of the migrate-incoming command (Yury Kotov)
- multifd: Initialize local variable (Juan Quintela)
- multifd: Be consistent about using uint64_t (Juan Quintela)
- Bug #1829242 correction. (Alexey Romko)
- migration/multifd: fix destroyed mutex access in terminating multifd threads (Jiahui Cen)
- migration/multifd: fix nullptr access in terminating multifd threads (Jiahui Cen)
- migration/multifd: not use multifd during postcopy (Wei Yang)
- migration/multifd: clean pages after filling packet (Wei Yang)
- migration: Make sure that we don't call write() in case of error (Juan Quintela)
- migration: fix multifd_send_pages() next channel (Laurent Vivier)
- migration/multifd: bypass uuid check for initial packet (Elena Ufimtseva) [Orabug: 32610480]
- migration/tls: add error handling in multifd_tls_handshake_thread (Hao Wang)
- migration/tls: fix inverted semantics in multifd_channel_connect (Hao Wang)
- migration/multifd: do not access uninitialized multifd_recv_state (Elena Ufimtseva) [Orabug: 32795384]
- io/channel-tls.c: make qio_channel_tls_shutdown thread-safe (Lukas Straub)
- qemu.spec: Enable qemu-guest-agent RPM for OL7 (Karl Heubaum) [Orabug: 32415543]
- virtio-net: Set mac address to hardware if the peer is vdpa (Cindy Lu)
- net: Add vhost-vdpa in show_netdevs() (Cindy Lu)
- vhost-vdpa: Add qemu_close in vhost_vdpa_cleanup (Cindy Lu)
- hw/virtio/vhost-vdpa: Fix Coverity CID 1432864 (Philippe Mathieu-Daudé)
- vhost-vdpa: negotiate VIRTIO_NET_F_STATUS with driver (Si-Wei Liu)
- configure: Fix build dependencies with vhost-vdpa. (Laurent Vivier)
- configure: simplify vhost condition with Kconfig (Marc-André Lureau)
- vhost-vdpa: add trace-events (Laurent Vivier)
- dma/pl330: Fix qemu_hexdump() usage in pl330.c (Mark Kanda)
- util/hexdump: introduce qemu_hexdump_line() (Laurent Vivier)
- util/hexdump: Reorder qemu_hexdump() arguments (Philippe Mathieu-Daudé)
- util/hexdump: Convert to take a void pointer argument (Philippe Mathieu-Daudé)
- net/colo-compare.c: Only hexdump packets if tracing is enabled (Lukas Straub)
- vhost-vdpa: batch updating IOTLB mappings (Jason Wang)
- vhost: switch to use IOTLB v2 format (Jason Wang)
- vhost-vdpa: remove useless variable (Laurent Vivier)
- virtio: vdpa: omit check return of g_malloc (Li Qiang)
- vhost-vdpa: fix indentation in vdpa_ops (Stefano Garzarella)
- virtio-net: check the existence of peer before accessing vDPA config (Jason Wang)
- virtio-pci: fix wrong index in virtio_pci_queue_enabled (Yuri Benditovich)
- virtio-pci: fix virtio_pci_queue_enabled() (Laurent Vivier)
- vhost-vdpa :Fix Coverity CID 1430270 / CID 1420267 (Cindy Lu)
- vhost-vdpa: fix the compile issue without kvm (Cindy Lu)
- vhost-vdpa: introduce vhost-vdpa net client (Cindy Lu)
- vhost-vdpa: introduce vhost-vdpa backend (Cindy Lu)
- linux headers: sync to 5.9-rc4 (Jason Wang)
- Linux headers: update (Cornelia Huck)
- virtio-net: fix rsc_ext compat handling (Cornelia Huck)
- linux-headers: update against Linux 5.7-rc3 (Cornelia Huck)
- linux-headers: update (Cornelia Huck)
- virtiofsd: Pull in kernel's fuse.h (Dr. David Alan Gilbert)
- linux-headers: Update (Bharata B Rao)
- linux-headers: Update (Greg Kurz)
- vhost_net: introduce set_config & get_config (Cindy Lu)
- vhost: implement vhost_force_iommu method (Cindy Lu)
- vhost: introduce new VhostOps vhost_force_iommu (Cindy Lu)
- vhost: implement vhost_vq_get_addr method (Cindy Lu)
- vhost: introduce new VhostOps vhost_vq_get_addr (Cindy Lu)
- vhost: implement vhost_dev_start method (Cindy Lu)
- vhost: introduce new VhostOps vhost_dev_start (Cindy Lu)
- vhost: check the existence of vhost_set_iotlb_callback (Jason Wang)
- virtio-pci: implement queue_enabled method (Jason Wang)
- virtio-bus: introduce queue_enabled method (Jason Wang)
- vhost_net: use the function qemu_get_peer (Cindy Lu)
- net: introduce qemu_get_peer (Cindy Lu)
- vhost: correctly turn on VIRTIO_F_IOMMU_PLATFORM (Jason Wang)
- imx7-ccm: add digprog mmio write method (Prasad J Pandit) [Orabug: 31576552] {CVE-2020-15469}
- tz-ppc: add dummy read/write methods (Prasad J Pandit) [Orabug: 31576552] {CVE-2020-15469}
- spapr_pci: add spapr msi read method (Prasad J Pandit) [Orabug: 31576552] {CVE-2020-15469}
- nvram: add nrf51_soc flash read method (Prasad J Pandit) [Orabug: 31576552] {CVE-2020-15469}
- prep: add ppc-parity write method (Prasad J Pandit) [Orabug: 31576552] {CVE-2020-15469}
- vfio: add quirk device write method (Prasad J Pandit) [Orabug: 31576552] {CVE-2020-15469}
- pci-host: designware: add pcie-msi read method (Prasad J Pandit) [Orabug: 31576552] {CVE-2020-15469}
- hw/pci-host: add pci-intack write method (Prasad J Pandit) [Orabug: 31576552] {CVE-2020-15469}
- oslib-posix: take lock before qemu_cond_broadcast (Bauerchen) [Orabug: 32555402]
- oslib-posix: initialize mutex and condition variable (Paolo Bonzini) [Orabug: 32555402]
- mem-prealloc: optimize large guest startup (Bauerchen) [Orabug: 32555402]
- i386: Add the support for AMD EPYC 3rd generation processors (Babu Moger)
- acpi: cpuhp: document CPHP_GET_CPU_ID_CMD command (Igor Mammedov)
- acpi: cpuhp: add CPHP_GET_CPU_ID_CMD command (Igor Mammedov)
- acpi: cpuhp: spec: add typical usecases (Igor Mammedov)
- acpi: cpuhp: spec: clarify store into 'Command data' when 'Command field' == 0 (Igor Mammedov)
- acpi: cpuhp: spec: fix 'Command data' description (Igor Mammedov)
- acpi: cpuhp: spec: clarify 'CPU selector' register usage and endianness (Igor Mammedov)
- acpi: cpuhp: introduce 'Command data 2' field (Igor Mammedov)
- x86: ich9: let firmware negotiate 'CPU hot-unplug with SMI' feature (Igor Mammedov)
- x86: ich9: factor out "guest_cpu_hotplug_features" (Igor Mammedov)
- x86: acpi: let the firmware handle pending "CPU remove" events in SMM (Igor Mammedov)
- x86: acpi: introduce AcpiPmInfo::smi_on_cpu_unplug (Igor Mammedov)
- acpi: cpuhp: introduce 'firmware performs eject' status/control bits (Igor Mammedov)
- x68: acpi: trigger SMI before sending hotplug Notify event to OSPM (Igor Mammedov)
- x86: acpi: introduce the PCI0.SMI0 ACPI device (Igor Mammedov)
- x86: acpi: introduce AcpiPmInfo::smi_on_cpuhp (Igor Mammedov)
- x86: ich9: expose "smi_negotiated_features" as a QOM property (Igor Mammedov)
- tests: acpi: mark to be changed tables in bios-tables-test-allowed-diff (Igor Mammedov)
- acpi: add aml_land() and aml_break() primitives (Igor Mammedov)
- x86: cpuhp: refuse cpu hot-unplug request earlier if not supported (Igor Mammedov)
- x86: cpuhp: prevent guest crash on CPU hotplug when broadcast SMI is in use (Igor Mammedov)
- x86: lpc9: let firmware negotiate 'CPU hotplug with SMI' features (Igor Mammedov)
- q35: implement 128K SMRAM at default SMBASE address (Igor Mammedov)
- hw/intc/arm_gic: Fix interrupt ID in GICD_SGIR register (Philippe Mathieu-Daudé) [Orabug: 32470471] {CVE-2021-20221}
- memory: clamp cached translation in case it points to an MMIO region (Paolo Bonzini) [Orabug: 32252673] {CVE-2020-27821}
- hw/sd/sdhci: Fix DMA Transfer Block Size field (Philippe Mathieu-Daudé) [Orabug: 32613470] {CVE-2021-3409}

* Wed Mar 31 2021 Karl Heubaum <karl.heubaum@oracle.com> - 4.2.1-6.el8
- i386/pc: Keep PCI 64-bit hole within usable IOVA space (Joao Martins)
- pc/cmos: Adjust CMOS above 4G memory size according to 1Tb boundary (Joao Martins)
- i386/pc: Round up the hotpluggable memory within valid IOVA ranges (Joao Martins)
- i386/pc: Account IOVA reserved ranges above 4G boundary (Joao Martins)

* Thu Mar 11 2021 Karl Heubaum <karl.heubaum@oracle.com> - 4.2.1-5.el8
- hostmem: fix default "prealloc-threads" count (Mark Kanda)
- hostmem: introduce "prealloc-threads" property (Igor Mammedov)
- qom: introduce object_register_sugar_prop (Paolo Bonzini)
- migration/multifd: Do error_free after migrate_set_error to avoid memleaks (Pan Nengyuan)
- multifd/tls: fix memoryleak of the QIOChannelSocket object when cancelling migration (Chuan Zheng)
- migration/multifd: fix hangup with TLS-Multifd due to blocking handshake (Chuan Zheng)
- migration/tls: add trace points for multifd-tls (Chuan Zheng)
- migration/tls: add support for multifd tls-handshake (Chuan Zheng)
- migration/tls: extract cleanup function for common-use (Chuan Zheng)
- migration/multifd: fix memleaks in multifd_new_send_channel_async (Pan Nengyuan)
- migration/multifd: fix nullptr access in multifd_send_terminate_threads (Zhimin Feng)
- migration/tls: add tls_hostname into MultiFDSendParams (Chuan Zheng)
- migration/tls: extract migration_tls_client_create for common-use (Chuan Zheng)
- migration/tls: save hostname into MigrationState (Chuan Zheng)
- tests/qtest: add a test case for pvpanic-pci (Mihai Carabas)
- pvpanic : update pvpanic spec document (Mihai Carabas)
- hw/misc/pvpanic: add PCI interface support (Mihai Carabas)
- hw/misc/pvpanic: split-out generic and bus dependent code (Mihai Carabas)
- qemu-img: Add --target-is-zero to convert (David Edmondson)
- 9pfs: Fully restart unreclaim loop (CVE-2021-20181) (Greg Kurz) [Orabug: 32441198] {CVE-2021-20181}
- ide: atapi: check logical block address and read size (CVE-2020-29443) (Prasad J Pandit) [Orabug: 32393835] {CVE-2020-29443}
- Document CVE-2019-20808 as fixed (Mark Kanda) [Orabug: 32339196] {CVE-2019-20808}
- block/iscsi:fix heap-buffer-overflow in iscsi_aio_ioctl_cb (Chen Qun) [Orabug: 32339207] {CVE-2020-11947}
- net: remove an assert call in eth_get_gso_type (Prasad J Pandit) [Orabug: 32102583] {CVE-2020-27617}
- nvdimm: honor -object memory-backend-file, readonly=on option (Stefan Hajnoczi) [Orabug: 32265408]
- hostmem-file: add readonly=on|off option (Stefan Hajnoczi) [Orabug: 32265408]
- memory: add readonly support to memory_region_init_ram_from_file() (Stefan Hajnoczi) [Orabug: 32265408]

* Thu Jan 14 2021 Karl Heubaum <karl.heubaum@oracle.com> - 4.2.1-4.el8
- Document CVE-2020-25723 as fixed (Mark Kanda) [Orabug: 32222397] {CVE-2020-25723}
- hw/net/e1000e: advance desc_offset in case of null descriptor (Prasad J Pandit) [Orabug: 32217517] {CVE-2020-28916}
- i386: Add 2nd Generation AMD EPYC processors (Babu Moger) [Orabug: 32217570]
- libslirp: Update version to include CVE fixes (Mark Kanda) [Orabug: 32208456] [Orabug: 32208462] {CVE-2020-29129} {CVE-2020-29130}
- Document CVE-2020-25624 as fixed (Mark Kanda) [Orabug: 32212527] {CVE-2020-25624}
- pvpanic: Advertise the PVPANIC_CRASHLOADED event support (Paolo Bonzini) [Orabug: 32102853]
- ati: check x y display parameter values (Prasad J Pandit) [Orabug: 32108251] {CVE-2020-27616}
- Add AArch64 support for QMP regdump tool and sosreport plugin (Mark Kanda) [Orabug: 32080658]
- Add qemu_regdump sosreport plugin support for '-mon' QMP sockets (Mark Kanda)
- migration/dirtyrate: present dirty rate only when querying the rate has completed (Chuan Zheng)
- migration/dirtyrate: record start_time and calc_time while at the measuring state (Chuan Zheng)
- migration/dirtyrate: Add trace_calls to make it easier to debug (Chuan Zheng)
- migration/dirtyrate: Implement qmp_cal_dirty_rate()/qmp_get_dirty_rate() function (Chuan Zheng)
- migration/dirtyrate: Implement calculate_dirtyrate() function (Chuan Zheng)
- migration/dirtyrate: Implement set_sample_page_period() and is_sample_period_valid() (Chuan Zheng)
- migration/dirtyrate: skip sampling ramblock with size below MIN_RAMBLOCK_SIZE (Chuan Zheng)
- migration/dirtyrate: Compare page hash results for recorded sampled page (Chuan Zheng)
- migration/dirtyrate: Record hash results for each sampled page (Chuan Zheng)
- migration/dirtyrate: move RAMBLOCK_FOREACH_MIGRATABLE into ram.h (Chuan Zheng)
- migration/dirtyrate: Add dirtyrate statistics series functions (Chuan Zheng)
- migration/dirtyrate: Add RamblockDirtyInfo to store sampled page info (Chuan Zheng)
- migration/dirtyrate: add DirtyRateStatus to denote calculation status (Chuan Zheng)
- migration/dirtyrate: setup up query-dirtyrate framwork (Chuan Zheng)
- ram_addr: Split RAMBlock definition (Juan Quintela)

* Tue Sep 29 2020 Karl Heubaum <karl.heubaum@oracle.com> - 4.2.1-3.el8
- qemu-kvm.spec: Install block storage module RPMs by default (Karl Heubaum) [Orabug: 31943789]
- qemu-kvm.spec: Enable block-ssh module RPM (Karl Heubaum) [Orabug: 31943763]
- hw: usb: hcd-ohci: check for processed TD before retire (Prasad J Pandit) [Orabug: 31901690] {CVE-2020-25625}
- hw: usb: hcd-ohci: check len and frame_number variables (Prasad J Pandit) [Orabug: 31901690] {CVE-2020-25625}
- hw: ehci: check return value of 'usb_packet_map' (Li Qiang) [Orabug: 31901649] {CVE-2020-25084}
- hw: xhci: check return value of 'usb_packet_map' (Li Qiang) [Orabug: 31901649] {CVE-2020-25084}
- usb: fix setup_len init (CVE-2020-14364) (Gerd Hoffmann) [Orabug: 31848849] {CVE-2020-14364}
- Document CVE-2020-12829 and CVE-2020-14415 as fixed (Mark Kanda) [Orabug: 31855502] [Orabug: 31855427] {CVE-2020-12829} {CVE-2020-14415}

* Mon Aug 31 2020 Karl Heubaum <karl.heubaum@oracle.com> - 4.2.1-2.el8
- hw/net/xgmac: Fix buffer overflow in xgmac_enet_send() (Mauro Matteo Cascella) [Orabug: 31667649] {CVE-2020-15863}
- hw/net/net_tx_pkt: fix assertion failure in net_tx_pkt_add_raw_fragment() (Mauro Matteo Cascella) [Orabug: 31737809] {CVE-2020-16092}
- migration: fix memory leak in qmp_migrate_set_parameters (Zheng Chuan) [Orabug: 31806256]
- virtio-net: fix removal of failover device (Juan Quintela) [Orabug: 31806255]
- pvpanic: introduce crashloaded for pvpanic (zhenwei pi) [Orabug: 31677154]

* Wed Jul 22 2020 Karl Heubaum <karl.heubaum@oracle.com> - 4.2.1-1.el8
- hw/sd/sdcard: Do not switch to ReceivingData if address is invalid (Philippe Mathieu-Daudé)  [Orabug: 31414336]  {CVE-2020-13253}
- hw/sd/sdcard: Update coding style to make checkpatch.pl happy (Philippe Mathieu-Daudé)  [Orabug: 31414336]
- hw/sd/sdcard: Do not allow invalid SD card sizes (Philippe Mathieu-Daudé)  [Orabug: 31414336]  {CVE-2020-13253}
- hw/sd/sdcard: Simplify realize() a bit (Philippe Mathieu-Daudé)  [Orabug: 31414336]
- hw/sd/sdcard: Restrict Class 6 commands to SCSD cards (Philippe Mathieu-Daudé)  [Orabug: 31414336]
- libslirp: Update to v4.3.1 to fix CVE-2020-10756 (Karl Heubaum)  [Orabug: 31604999]  {CVE-2020-10756}
- Document CVEs as fixed 2/2 (Karl Heubaum)  [Orabug: 30618035]  {CVE-2017-18043} {CVE-2018-10839} {CVE-2018-11806} {CVE-2018-12617} {CVE-2018-15746} {CVE-2018-16847} {CVE-2018-16867} {CVE-2018-17958} {CVE-2018-17962} {CVE-2018-17963} {CVE-2018-18849} {CVE-2018-19364} {CVE-2018-19489} {CVE-2018-3639} {CVE-2018-5683} {CVE-2018-7550} {CVE-2018-7858} {CVE-2019-12068} {CVE-2019-15034} {CVE-2019-15890} {CVE-2019-20382} {CVE-2020-10702} {CVE-2020-10761} {CVE-2020-11102} {CVE-2020-11869} {CVE-2020-13361} {CVE-2020-13765} {CVE-2020-13800} {CVE-2020-1711} {CVE-2020-1983} {CVE-2020-8608}
- Document CVEs as fixed 1/2 (Karl Heubaum)  [Orabug: 30618035]  {CVE-2017-10806} {CVE-2017-11334} {CVE-2017-12809} {CVE-2017-13672} {CVE-2017-13673} {CVE-2017-13711} {CVE-2017-14167} {CVE-2017-15038} {CVE-2017-15119} {CVE-2017-15124} {CVE-2017-15268} {CVE-2017-15289} {CVE-2017-16845} {CVE-2017-17381} {CVE-2017-18030} {CVE-2017-2630} {CVE-2017-2633} {CVE-2017-5715} {CVE-2017-5753} {CVE-2017-5754} {CVE-2017-5931} {CVE-2017-6058} {CVE-2017-7471} {CVE-2017-7493} {CVE-2017-8112} {CVE-2017-8309} {CVE-2017-8379} {CVE-2017-8380} {CVE-2017-9503} {CVE-2017-9524} {CVE-2018-12126} {CVE-2018-12127} {CVE-2018-12130} {CVE-2018-16872} {CVE-2018-20123} {CVE-2018-20124} {CVE-2018-20125} {CVE-2018-20126} {CVE-2018-20191} {CVE-2018-20216} {CVE-2018-20815} {CVE-2019-11091} {CVE-2019-12155} {CVE-2019-14378} {CVE-2019-3812} {CVE-2019-5008} {CVE-2019-6501} {CVE-2019-6778} {CVE-2019-8934} {CVE-2019-9824}
- qemu-kvm.spec: Add .spec file for OL8 (Karl Heubaum)  [Orabug: 30618035]
- qemu.spec: Add .spec file for OL7 (Karl Heubaum)  [Orabug: 30618035]
- qemu-submodule-init: Add Git submodule init script (Karl Heubaum)  [Orabug: 30618035]
- vhost.conf: Initial vhost.conf (Karl Heubaum)  [Orabug: 30618035]
- parfait: Add buildrpm/parfait-qemu.conf (Karl Heubaum)  [Orabug: 30618035]
- virtio: Set PCI subsystem vendor ID to Oracle (Karl Heubaum)  [Orabug: 30618035]
- qemu_regdump.py: Initial qemu_regdump.py (Karl Heubaum)  [Orabug: 30618035]
- qmp-regdump: Initial qmp-regdump (Karl Heubaum)  [Orabug: 30618035]
- bridge.conf: Initial bridge.conf (Karl Heubaum)  [Orabug: 30618035]
- kvm.conf: Initial kvm.conf (Karl Heubaum)  [Orabug: 30618035]
- 80-kvm.rules: Initial 80-kvm.rules (Karl Heubaum)  [Orabug: 30618035]
- exec: set map length to zero when returning NULL (Prasad J Pandit)  [Orabug: 31439733]  {CVE-2020-13659}
- megasas: use unsigned type for reply_queue_head and check index (Prasad J Pandit)  [Orabug: 31414338]  {CVE-2020-13362}
- memory: Revert "memory: accept mismatching sizes in memory_region_access_valid" (Michael S. Tsirkin)  [Orabug: 31439736] [Orabug: 31452202]  {CVE-2020-13754} {CVE-2020-13791}

* Wed Jul 22 2020 Karl Heubaum <karl.heubaum@oracle.com> - 4.1.1-3.el8
- buildrpm/spec files: Don't package elf2dmp (Karl Heubaum)  [Orabug: 31657424]
- qemu-kvm.spec: Enable the block-curl package (Karl Heubaum)  [Orabug: 31657424]
- qemu.spec: enable have_curl in spec (Dongli Zhang)  [Orabug: 31657424]

* Wed Jun 10 2020 Karl Heubaum <karl.heubaum@oracle.com> - 4.1.1-2.el8
- Document CVE-2020-13765 as fixed (Karl Heubaum)  [Orabug: 31463250]  {CVE-2020-13765}
- kvm: Reallocate dirty_bmap when we change a slot (Dr. David Alan Gilbert)  [Orabug: 31076399]
- kvm: split too big memory section on several memslots (Igor Mammedov)  [Orabug: 31076399]
- target/i386: do not set unsupported VMX secondary execution controls (Vitaly Kuznetsov)  [Orabug: 31463710]
- target/i386: add VMX definitions (Paolo Bonzini)  [Orabug: 31463710]
- ati-vga: check mm_index before recursive call (CVE-2020-13800) (Prasad J Pandit)  [Orabug: 31452206]  {CVE-2020-13800}
- es1370: check total frame count against current frame (Prasad J Pandit)  [Orabug: 31463235]  {CVE-2020-13361}
- ati-vga: Fix checks in ati_2d_blt() to avoid crash (BALATON Zoltan)  [Orabug: 31238432]  {CVE-2020-11869}
- libslirp: Update to stable-4.2 to fix CVE-2020-1983 (Karl Heubaum)  [Orabug: 31241227]  {CVE-2020-1983}
- Document CVEs as fixed (Karl Heubaum)   {CVE-2019-12068} {CVE-2019-15034}
- libslirp: Update to version 4.2.0 to fix CVEs (Karl Heubaum)  [Orabug: 30274592] [Orabug: 30869830]  {CVE-2019-15890} {CVE-2020-8608}
- target/i386: add support for MSR_IA32_TSX_CTRL (Paolo Bonzini)  [Orabug: 31124041]
- qemu-img: Add --target-is-zero to convert (David Edmondson)
- vnc: fix memory leak when vnc disconnect (Li Qiang)  [Orabug: 30996427]  {CVE-2019-20382}
- iscsi: Cap block count from GET LBA STATUS (CVE-2020-1711) (Felipe Franciosi)  [Orabug: 31124035]  {CVE-2020-1711}
- qemu.spec: Remove "BuildRequires: kernel" (Karl Heubaum)  [Orabug: 31124047]
